var exec = require('child_process').spawn,
    path = require('path');

module.exports = function(grunt) {

    grunt.initConfig({
        pkg: grunt.file.readJSON('package.json'),

        connect: {
            server: {
                options: {
                    hostname: '*',
                    port: 9003,
                    keepalive: true
                }
            }
        },

        jshint: {
            options: {
                asi:          false,
                bitwise:      true,
                boss:         false,
                browser:      true,
                curly:        true,
                debug:        false,
                eqeqeq:       true,
                eqnull:       false,
                es5:          false,
                esnext:       false,
                evil:         false,
                expr:         true,
                forin:        false,
                funcscope:    false,
                globalstrict: false,
                immed:        true,
                iterator:     false,
                lastsemic:    false,
                latedef:      true,
                laxbreak:     true,
                laxcomma:     false,
                loopfunc:     false,
                maxerr:       500,
                maxlen:       0,
                multistr:     false,
                newcap:       true,
                noarg:        true,
                node:         true,
                noempty:      true,
                nomen:        false,
                onecase:      false,
                onevar:       true,
                passfail:     false,
                plusplus:     false,
                proto:        false,
                regexdash:    false,
                regexp:       false,
                scripturl:    false,
                shadow:       false,
                smarttabs:    false,
                strict:       true,
                sub:          false,
                supernew:     false,
                trailing:     true,
                unused:       'vars',
                white:        false,
                yui:          true
            },
            post: {
                src: [
                    'build/**/*.js',
                    '!build/**/*-coverage.js',
                    '!build/**/*-debug.js',
                    '!build/**/*-min.js',
                    '!build/**/*-templates.js'
                ]
            },
            pre: {
                options: {
                    strict: false
                },
                src: [
                    'src/**/*.js{on,}',
                    '!src/**/tmpl/*.js'
                    ]
            }
        },

        watch: {
            src: {
                files: [
                    'src/**/*.js',
                    'src/**/*.css',
                    'src/**/build.json',
                    '!src/**/tmpl/*.js'
                ],
                tasks: [
                    'yui_template',
                    'jshint:pre',
                    'modules',
                    'shifter',
                    'grover'
                ],
                options: {
                    spawn: false
                }
            },
            templates: {
                files: ['src/**/tmpl/*.html'],
                tasks: [
                    'yui_template',
                    'grover'
                ],
                options: {
                    spawn: false
                }
            },
            tests: {
                files: ['src/**/tests/unit/**/.{html,js}'],
                tasks: ['grover'],
                options: {
                    spawn: false
                }
            }
        },

        shifter: {
            options: {
                bin: '/usr/bin/shifter',
                strict: true,
                lint: false,
                csslint: false,
                silent: true
            }
        },

        grover: {
            options: {
                bin: '/usr/bin/grover'
            },
            unit: {
                src: ['./src/**/tests/unit/*.html']
            }
        },

        modules: {
            u1: {
                src: ['./src/**/build.json'],
                dest: './build/config.js',
                base: '/convoy'
            }
        },

        clean: ['./build', './release'],

        compress: {
            release: {
                options: {
                    archive: 'release/<%= pkg.version %>/<%= pkg.name %>-<%= pkg.version %>.tgz',
                    mode: 'tgz'
                },
                src: ['build/**']
            }
        },

        yui_template: {
            all: {
                files: [{
                    expand: true,
                    src: 'src/**/tmpl/*-{micro,hbs}.html',
                    ext: '.js'
                }]
            }
        }

    });

    /*
     * make sure watch only acts on the changed files and their
     * associated files
     */
    grunt.event.on('watch', function(action, filepath) {


        var module = filepath.split(path.sep)[1];


        // jshint
        grunt.config('jshint.pre.src', filepath);

        // yui_template
        grunt.config('yui_template.all.src', filepath);

        // shifter
        grunt.config('shifter.options.modules', [module]);
        grunt.config('shifter.options.silent', false);

        // grover
        grunt.config('grover.unit.src', './src/' + module + '/tests/unit/*.html');

    });

    grunt.loadNpmTasks('grunt-contrib-clean');
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-contrib-compress');
    grunt.loadNpmTasks('grunt-contrib-connect');
    grunt.loadNpmTasks('grunt-yui-template-compile');

    grunt.registerTask('build', ['clean', 'yui_template', 'shifter', 'modules', 'jshint:post']);
    grunt.registerTask('test', ['build', 'grover']);

    grunt.registerTask('shifter', 'Build YUI modules with Shifter', function() {
        var done = this.async(),
            config = grunt.config.get('shifter.options') || {},
            shifter = config.bin || '/usr/bin/shifter',
            args = [
                shifter,
                'build',
                '--walk'
            ],
            child;

        if (config) {
            // --modules
            if (config.modules && config.modules.length) {
                config.modules.forEach(function(module) {
                    args = args.concat('--modules', module);
                });
            }

            if (config.strict) {
                args.push('--strict');
            }

            if (!config.nolint) {
                args.push('--no-lint');
            }

            if (!config.nocsslint) {
                args.push('--no-csslint');
            }

            if (config.quiet) {
                args.push('--quiet');
            }

            if (config.silent) {
                args.push('--silent');
            }
        }

        child = exec(process.execPath, args, {
            cwd: path.join(process.cwd(), 'src'),
            stdio: 'inherit',
            env: process.env
        });

        child.on('exit', function(code) {
            if (code) {
                grunt.fail.fatal('shifter build exited with code: ' + code);
            }
            done();
        });
    });

    grunt.registerMultiTask('grover', 'Test with Grover', function() {
        var done = this.async(),
        // FIXME hard coded path path
            grover = this.data.bin || '/usr/bin/grover',
            args,
            child;

        // arguments are a list of files to test
        args = this.filesSrc;
        args.unshift(grover);

        console.log(this.files[0].src);

        child = exec(process.execPath, args, {
            cwd: process.cwd(),
            stdio: 'inherit',
            env: process.env
        });

        child.on('exit', function(code) {
            if (code) {
                grunt.fail.fatal('grover build exited with code: ' + code);
            }
            done();
        });
    });

    grunt.registerMultiTask('modules', 'Build modules hash for YUI', function() {

        var modules = {},
            config = {},
            out = '',
            group = this.target,
            base = this.data.base,
            template = this.data.tmpl ||
                'YUI.GlobalConfig = <%= config %>;';

        if (!base) {
            throw new Error('need base path for YUI config');
        }

        this.filesSrc.forEach(function(filepath) {
            var meta = grunt.file.readJSON(filepath),
                key;

            // allow for multiple modules per build file
            for (key in meta.builds) {

                if (meta.builds.hasOwnProperty(key)) {
                    modules[key] = {};
                    modules[key] = meta.builds[key].config;

                    if (meta.builds[key].skinnable) {
                        modules[key].skinnable = meta.builds[key].skinnable;
                    }
                }

            }
            for (key in meta.rollups) {

                if (meta.rollups.hasOwnProperty(key)) {
                    modules[key] = {};
                    modules[key] = meta.rollups[key].config;
                }

            }
        });

        config.groups = {};
        config.groups[group] = {};
        config.groups[group].base = base;
        config.groups[group].modules = modules;

        out = grunt.template.process(template, {data: {config: JSON.stringify(config) }});

        grunt.file.write(this.data.dest, out);

    });

};
