
var FilesUploader,

    BINDUI          = 'bindUI',
    RENDERUI        = 'renderUI',

    START           = 'start',
    SKIP            = 'skip',
    STATUS          = 'status',
    FILES           = 'files',
    CANCEL          = 'cancel',
    DISMISS         = 'yui3-dismiss',
    CONTROLS        = 'controls',
    FILECANCEL      = 'file-cancel',
    HEADER          = 'header',
    NOFILES         = 'nofiles',
    UPLOADERWRAPPER = 'uploader-wrapper',
    PROGRESS        = 'progress',
    RETRY           = 'retry',

    CLASSES         = [ CONTROLS, HEADER, FILES, FILECANCEL, NOFILES, UPLOADERWRAPPER, PROGRESS, CANCEL, START, RETRY, STATUS, SKIP ],
    CLS             = {};

/**
* Extends uploader-html5 with a frontend and functionality.
*
* @module one-uploader
*/
FilesUploader = Y.Base.create('oneFilesUploader', Y.U1.Widgets.Uploader, [Y.U1.Widgets.UploaderFollow], {

    /**
    * The wrapper for all files nodes
    *
    * @property _filesNode
    */
    _filesNode: null,

    /**
    * The node shown when there are no files
    *
    * @property _noFilesNode
    */
    _noFilesNode: null,

    /**
    * The node for status
    *
    * @property _statusNodes
    */
    _statusNodes: null,

    /**
    * Header node
    *
    * @property _headerNode
    */
    _headerNode: null,

    /**
    * Controls node
    *
    * @property _controlsNode
    */
    _controlsNode: null,

    _setupClasses: function () {

        Y.Array.each(CLASSES, function (c) {
            CLS[c] = this.getClassName(c);
        }, this);

        return CLS;

    },

    initializer: function () {

        this._setupClasses();

        Y.before(this._renderFilesUI, this, RENDERUI);
        Y.before(this._bindFilesUI, this, BINDUI);

        this.after('render', function (){

            // move the selectFilesButton
            // nuke the default style
            // TODO: override template for this instance?
            this._controlsNode.prepend(this.get('selectFilesButton').remove().setAttribute('style', ''));

        }, this);

    },

    destructor: function () {

        this.get('simpleUploader').show();

    },

    /**
    * Renders the ui
    *
    * @method _renderFilesUI
    */
    _renderFilesUI: function () {

        // dom structure
        this._renderBase();

        this.get('simpleUploader').hide();

    },

    /**
    * Appends a skeleton to the contentBox
    * Will also throw Error if something important is missing
    *
    * @method _renderBase
    * @protected
    */
    _renderBase: function () {

        var cb = this.get('contentBox');

        cb.append(FilesUploader.TEMPLATES.base({
            headerClass:            CLS[HEADER],
            filesClass:             CLS[FILES],
            uploaderWrapperClass:   CLS[UPLOADERWRAPPER],
            statusClass:            CLS[STATUS],
            controlsClass:          CLS[CONTROLS],
            startClass:             CLS[START],
            cancelClass:            CLS[CANCEL],
            noFilesClass:           CLS[NOFILES],
            noFilesSelectedLabel:   this.get('strings.noFilesSelected'),
            cancelLabel:            this.get('strings.cancel'),
            uploadLabel:            this.get('strings.upload')
        }));

        this._filesNode     = cb.one('.'+CLS[FILES]);

        if(!this._filesNode) {
            throw new Error('Missing files container');
        }

        this._noFilesNode   = cb.one('.'+CLS[NOFILES]);
        this._headerNode    = cb.one('.'+CLS[HEADER]);
        this._statusNodes   = {
            main:           cb.one('.'+CLS[STATUS] + '-vitals'),
            time:           cb.one('.'+CLS[STATUS] + '-timeremaining')
        };
        this._controlsNode  = cb.one('.'+CLS[CONTROLS]);

    },

    /**
    * Renders a file inside it's row
    * Uses `TEMPLATES.file`
    *
    * @param file {Y.File}
    * @param data {Object} Data for template rendering
    * @method renderFile
    * @public
    */
    renderFile: function (file, d) {

        var data = d || {};

        // add a node to the file for ease
        if(!file._node) {
            file._node = this.getFileNode(file);
        }

        // put css classes on this files node
        if(data.classes) {
            file._node.setAttribute('class', data.classes + " yui3-file");
        }

        // data to store on node
        if(data.dataAttrs && data.dataAttrs.length > 0) {
            Y.each(data.dataAttrs, function (d) {
                file._node.setData(d.name, d.val);
            });
        }

        // merge file data with template data
        data = Y.merge(data, {
            fileId:             file.get('id'),
            fileName:           file.get('name'),
            fileSize:           file._size || file.get('size'),
            fileCancelClass:    CLS[FILECANCEL]
        });

        file._node.setHTML(FilesUploader.TEMPLATES.file(data));

        this.fire('filerender', { file: file });
    },

    /**
    *
    * @method _bindFilesUI
    */
    _bindFilesUI: function () {

        var cb = this.get('contentBox');

        cb.delegate('click', this._filesCancelFileClick,    '.'+CLS[FILECANCEL], this);
        cb.delegate('click', this.start,                    '.'+CLS[START], this);
        cb.delegate('click', this.cancel,                   '.'+CLS[CANCEL], this);
        cb.delegate('click', this._filesRetryClick,         '.'+CLS[RETRY], this);
        cb.delegate('click', this._skipStalledUploads,      '.'+CLS[SKIP], this);
        cb.delegate('click', this._complete,                '.'+DISMISS, this);

        this.after('filelistrender',    this._filesAlignHeader);
        this.after('fileselect',        this._filesSelectFiles);

        this.on('enabledChange',        this._filesEnabledChange);
        this.on('fileListChange',       this._filesFileListChange);
        this.on('statusChange',         this._filesStatusChange);
        this.on('dropAreaStatusChange', this._filesDropAreaStatusChange);

        this.on('uploadprogress',       this._filesUploadProgress);
        this.on('uploaderror',          this._filesUploadError);
        this.on('uploadcomplete',       this._filesUploadComplete);

    },

    /**
    * Handler for file retries
    *
    * @method _filesRetryClick
    * @protected
    */
    _filesRetryClick: function (e) {

        var id   = e.target.ancestor('li').getData('id');

        this.retryFile(this.getFile(id));

    },

    getFileNode: function (file) {

        return this.get('contentBox').one('#' + file.get('id') + '_row');

    },

    /**
    *
    * @method _filesEnabledChange
    * @protected
    */
    _filesEnabledChange: function (e) {
        this._controlsNode.one('.'+CLS[START]).set('disabled', !e.newVal);
        this.get('selectFilesButton').set('disabled', !e.newVal);

    },

    /**
    *
    * @method _filesFileListChange
    * @protected
    */
    _filesFileListChange: function (e) {

        if(e.newVal.length === 0) {
            this._noFilesNode.show();
        } else {
            this._noFilesNode.hide();
        }

        // if pending, allow user to upload
        if(!this.getHaveUploadsInProgress()) {
            this._controlsNode.one('.'+CLS[START]).set('disabled', e.newVal.length === 0);
        }

    },

    /**
    *
    * Updates status node whenever the status was changed
    *
    * @method _filesStatusChange
    * @protected
    */
    _filesStatusChange: function (e) {

        this._statusNodes.main.setHTML(e.newVal || this.get('status'));

    },

    _filesDropAreaStatusChange: function (e) {

        this._noFilesNode.one('span').set('text', e.newVal || this.get('dropAreaStatus'));

    },

    _filesAlignHeader: function () {

        // make header columns match up with file columns
        var headerWidth     = this._headerNode.get('clientWidth'),
            fileRow         = this.get('contentBox').one('.'+CLS[FILES]+' li:not(.'+CLS[NOFILES]+')'),
            fileRowWidth    = fileRow ? fileRow.get('clientWidth') : 0;

        if(fileRowWidth > 0) {
            this._headerNode.setStyle('paddingRight', (headerWidth-fileRowWidth) +'px');
        }

    },

    /**
    *
    *
    * @oaram event {Event}
    * @method _filesSelectFiles
    * @protected
    */
    _filesSelectFiles: function (event) {

        var files       = event.fileList,
            filesList   = this._filesNode.one('ul'),
            perFileVars = {};

        Y.each(files, function (fileInstance) {

            // set a file size for us
            fileInstance._size = this.getFriendlySize(fileInstance.get('size'));

            // render a file wrapper
            filesList.append(FilesUploader.TEMPLATES.fileRow({
                fileId: fileInstance.get('id')
            }));

            // render the file
            this.renderFile(fileInstance, {
                status: this.get('strings.not_started'),
                dataAttrs: [{
                    name:   'status',
                    val:    'pending'
                }]
            });

            perFileVars[fileInstance.get('id')] = {
                parent_key: this.get('current_folderid'),
                base: this.get('current_path'),
                path: this.get('path'),
                csrfmiddlewaretoken: this.get('csrfmiddlewaretoken'),
                redirect: this.get('redirect')
            };
        }, this);

        this.set("postVarsPerFile", Y.merge(this.get("postVarsPerFile"), perFileVars));

        this.fire('filelistrender');

    },


    /**
    * Handler for when a file is cancelled
    * Removes file from fileList, and queue if active. If in a queue it is also cancelled
    * Manually updates the uploader's total uploaded bytes
    *
    * @method _filesCancelFileClick
    * @protected
    */
    _filesCancelFileClick: function (e) {

        var fileRow     = e.target.ancestor('li'),
            file        = this.getFile(fileRow.getData('id'));

        fileRow.remove();

        this.cancelFile(file);

        this.fire('filelistrender');

    },


    /**
    * Handler for a file's progression
    * Changes the file markup's class and sets a status and progress var
    *
    * @oaram event {Event}
    * @method _filesUploadProgress
    * @protected
    */
    _filesUploadProgress: function (event) {

        this.renderFile(event.file, {
            classes: 'yui3-fileprogression',
            status: this.get('strings.not_started'),
            progress: event.percentLoaded,
            dataAttrs: [{
                name:   'status',
                val:    'uploading'
            }]
        });

    },


    /**
    * Handler for a file's error
    *
    * @oaram event {Event}
    * @method _filesUploadError
    * @protected
    */
    _filesUploadError: function (event) {

        var error = this.getFileError(event);

        this.renderFile(event.file, {
            classes:        'yui3-filefailed',
            status:         Y.Lang.sub(this.get('strings.error'), {
                                error: error.source
                            }),
            information:    error.message,
            retryAble:      true,
            retryClass:     CLS[RETRY],
            dataAttrs:      [{
                                name:   'status',
                                val:    'failed'
                            }],
            fail:           true
        });

    },

    /**
    * File complete handler
    *
    * @oaram event {Event}
    * @method _filesUploadComplete
    * @protected
    */
    _filesUploadComplete: function (event) {

        this.renderFile(event.file, {
            classes:    'yui3-filecomplete',
            status:     this.get('strings.upload_complete'),
            dataAttrs:  [{
                            name:   'status',
                            val:    'complete'
                        }],
            complete:   true
        });

    }


}, {
    ATTRS: {
        csrfmiddlewaretoken: { value: '' },
        current_folderid: { value: '' },
        current_path: { value: '' },
        multipleFiles: { value: true },
        path: { value: '' },
        redirect: { value: 'False' },
        simpleUploader: {
            value: null,
            setter: function (val) {
                return Y.Lang.isString(val) ? Y.one(val) : val;
            }
        }
    },

    TEMPLATES: {

        base: Y.Template.get('t-uploader-base'),
        file: Y.Template.get('t-uploader-file'),
        fileRow: Y.Template.get('t-uploader-filerow')

    }
});

Y.namespace('U1.Widgets').FilesUploader = FilesUploader;
