YUI.add('one-files-uploader-test', function (Y) {

var suite;

suite = new Y.Test.Suite('one-files-uploader');

suite.add(new Y.Test.Case({

    name: 'Base',

    setUp: function () {

        this.app = new Y.U1.Widgets.FilesUploader({
            simpleUploader: '.simple-uploader'
        });
        this.app.render('.uploader');

        this.filearray = [];
        for (var i = 0; i <= 2; i++) {
            this.filearray.push(new Y.FileHTML5({name: i + ".jpg", size: i*1000, type: "jpg"}));
        }

        this.queue = new Y.UploaderHTML5.Queue({
            simUploads: 1,
            //errorAction: Y.Uploader.Queue.RESTART_ASAP,
            fileList: this.filearray,
            uploadURL: "test.html"
        });

       // set filelist
        this.app.set('fileList', this.filearray);
        this.app.fire('fileselect', {
            fileList: this.filearray
        });

    },

    tearDown: function () {

        if(this.app) {
            this.app.destroy();
        }

        delete this.app;
        delete this.filearray;
        delete this.queue;
    },

    "instance ok": function () {
        Y.Assert.isInstanceOf(Y.U1.Widgets.FilesUploader, this.app);
    },

    "test getFileNode": function () {
        Y.Assert.areSame(this.filearray[0]._node, this.app.getFileNode(this.filearray[0]));
    },

    "cancel file from ui": function () {

        // cancel click
        this.app.getFileNode(this.filearray[0]).one('.'+this.app.getClassName('file-cancel')).simulate('click');

        Y.ArrayAssert.doesNotContain(this.filearray[0], this.app.get('fileList'));
        Y.ArrayAssert.contains(this.filearray[1], this.app.get('fileList'));

    },

    "cancel file from ui removed from queue": function () {

        // make queue active and push to queuedFiles
        this.app.queue = this.queue;
        this.app.queue._currentState = Y.UploaderHTML5.Queue.UPLOADING;
        this.app.uploadAll();

        // cancel click
        this.app.getFileNode(this.filearray[1]).one('.'+this.app.getClassName('file-cancel')).simulate('click');

        Y.ArrayAssert.doesNotContain(this.filearray[1], this.app.queue.queuedFiles);
        Y.ArrayAssert.contains(this.filearray[2], this.app.queue.queuedFiles);
    },

    "incomplete fired after all files cancelled": function () {
        var eventCalled = false;

        // make queue active and push to queuedFiles
        this.app.queue = this.queue;
        this.app.queue._currentState = Y.UploaderHTML5.Queue.UPLOADING;
        this.app.uploadAll();

        // this should be called, so let us just detach the defaul one
        // and create our own
        this.app.detach('incomplete');
        this.app.on('incomplete', function () {
            eventCalled = true;
        });

        // cancel all uploads while they are uploading
        Y.Array.each(this.filearray, function (file) {
            this.app.getFileNode(file).one('.'+this.app.getClassName('file-cancel')).simulate('click');
        }, this);


        Y.Assert.isTrue(eventCalled, 'incomplete event called');

    },

    "test _setupClasses": function () {

        var classes = this.app._setupClasses();

        Y.Assert.isObject(classes);
        Y.Assert.isTrue(Y.Object.size(classes) > 0);

    },

    "test renderFile": function () {

        var file = new Y.FileHTML5({name: 'foo.jpg', size: 1337, type: 'jpg'});
            node = Y.Node.create('<div></div>');

        file._node = node;

        this.app.renderFile(file);

        Y.Assert.areSame(Y.Lang.trim(file._node.one('.yui3-filename').get('text')), file.get('name'), 'name mismatch');

        this.app.renderFile(file, {
            classes: 'foo'
        });
        Y.Assert.isTrue(file._node.hasClass('foo'));

    },

    "test getFilesTotals": function () {
        var file1 = new Y.FileHTML5({name: 'foo.jpg', size: 21, type: 'jpg'}),
            file2 = new Y.FileHTML5({name: 'bar.jpg', size: 21, type: 'jpg'});

        this.app.set('fileList', [file1, file2]);

        Y.Assert.areSame(42, this.app.getFilesTotals().size);
        Y.Assert.areSame(2, this.app.getFilesTotals().count);
    },

    "pending uploads reported to user": function () {
        var cls             = this.app._setupClasses(),
            statusText      = this.app.get('contentBox').one('.' + cls.status).get('text'),

            undefinedRe     = new RegExp('undefined'),
            filesRe         = new RegExp(this.app.get('fileList').length + ' files');

        Y.Assert.isFalse(undefinedRe.test(statusText), 'undefined in status');
        Y.Assert.isTrue(filesRe.test(statusText), 'did not find amount of files');
    },

    "no NaNs in pending total progress node": function () {
        var zeroLengthFile      = new Y.FileHTML5({ name: 'zero.jpg', size: 0, type: 'jpg' }),
            nullLengthFile      = new Y.FileHTML5({ name: 'null.jpg', size: null, type: 'jpg' }),
            undefinedLengthFile = new Y.FileHTML5({ name: 'undefined.jpg', size: undefined, type: 'jpg' }),
            negativeLengthFile  = new Y.FileHTML5({ name: 'negative.jpg', size: -1, type: 'jpg' }),

            NaNre               = new RegExp('NaN');

        Y.each([zeroLengthFile, nullLengthFile, undefinedLengthFile, negativeLengthFile], function (file) {
            this.app.set('fileList', [file]);
            this.app.fire('fileselect');
            Y.Assert.isFalse(NaNre.test(this.app._statusNodes.main.get('text')));
        }, this);
    },

    "lacking a proper percentage, show something else": function () {
        var badRe               = new RegExp('NaN|null|undefined'),
            values              = [NaN, null, undefined, "NaN", "null", "undefined"];

        Y.each(values, function (val) {
            this.app.fire('totaluploadprogress', {
                percentLoaded: val
            });
            Y.Assert.isFalse(badRe.test(this.app._statusNodes.main.get('text')), 'no bad values in _totalProgressNode');
        }, this);

    },

    "test upload button disabled before files and after upload start": function () {

        var app = new Y.U1.Widgets.FilesUploader({
            simpleUploader: '.simple-uploader'
        });
        app.render();

        Y.Assert.isTrue(app.get('contentBox').one('.yui3-onefilesuploader-start').get('disabled'));

        app.set('fileList', this.filearray);
        app.fire('fileselect', {
            fileList: this.filearray
        });

        Y.Assert.isFalse(app.get('contentBox').one('.yui3-onefilesuploader-start').get('disabled'));

        app.set('enabled', false);

        Y.Assert.isTrue(app.get('contentBox').one('.yui3-onefilesuploader-start').get('disabled'));

        app.destroy();

    }


}));

suite.add(new Y.Test.Case({

    name: 'Template Scenarios',

    _render: function () {
        this.app = new Y.U1.Widgets.Uploader();
        this.app.render('.uploader');
    },

    _nuke: function (selector) {
        this.app.get('contentBox').one(selector).remove();
    },

    _pushAllButtonsPullAllLevers: function () {
        try {
            this.app.set('fileList', []);
            this.app.fire('fileselect');
            this.app.set('fileList', [new Y.FileHTML5({ name: 'zero.jpg', size: 0, type: 'jpg' })]);
            this.app.fire('fileselect');
            this.app.fire('totaluploadprogress', {
                percentLoaded: 5
            });
            this.app._skipStalledUploads();
            this.app._uploadStalled();
            this.app._complete();

        } catch (e) {
            Y.Assert.fail(e.toString());
        }

        Y.Assert.pass();
    },

    setUp: function () {

        Y.U1.Widgets.FilesUploader.TEMPLATES.base = Y.Template.Micro.revive(Y.U1.Templates['t-uploader-base']);
        Y.U1.Widgets.FilesUploader.TEMPLATES.file = Y.Template.Micro.revive(Y.U1.Templates['t-uploader-file']);
        Y.U1.Widgets.FilesUploader.TEMPLATES.fileRow = Y.Template.Micro.revive(Y.U1.Templates['t-uploader-filerow']);

    },

    tearDown: function () {
        if(this.app) {
            this.app.destroy();
            delete this.app;
        }
    },

    "no _filesNode will throw informative error": function () {
        Y.U1.Widgets.FilesUploader.TEMPLATES.base = function () {};

        this.app = new Y.U1.Widgets.FilesUploader({
            simpleUploader: '.simple-uploader'
        });
        Y.Assert.throwsError(new Error('Missing files container'), Y.bind(this.app.render, this.app));


    },

    "base template needs only files container and nofiles node > span": function () {

        Y.U1.Widgets.FilesUploader.TEMPLATES.base =
            Y.Template.Micro.compile('<div class="<%= data.filesClass %>"><div class="<%= data.noFilesClass %>"><span></span></div></div>');

        this._render();
        this._pushAllButtonsPullAllLevers();
    }

}));

Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: ['one-files-uploader']
});
