YUI.add('one-music-views-test', function (Y) {

var suite,

    setupTemplate = function () {

        // creates template in which our created music views will live and die
        Y.one('.container').append(Y.Node.create(Y.Template.Micro.render(Y.one('#template').getContent(), {
            songs: [
                { sID: 'bar' },
                { sID: 'baz', current: true }
            ]
        })));
    },

    tearDownTemplate = function () {
        Y.one('.container').get('childNodes').remove().destroy();
    },

    widgetConfig = {
        srcNode:        '.srcNode',
        contentBox:     '.contentBox'
    };

suite = new Y.Test.Suite('one-music-views');

suite.add(new Y.Test.Case({

    name: 'PlayerView',

    setUp: function () {

        var mockHistory     = Y.Mock();

        setupTemplate();

        this.module = new Y.Music.PlayerView(Y.merge(widgetConfig, {
            history:        mockHistory,
            index:          1
        }));
    },

    tearDown: function () {
        tearDownTemplate();


        if(this.module) {
            this.module.destroy();
            delete this.module;
        }
    },

    _getSongNode: function (sID) {
        return Y.one('[data-u1m-songid='+sID+']');
    },

    "instantiated ok": function () {
        Y.Assert.isInstanceOf(Y.Music.PlayerView, this.module);
    },

    "test test test template": function () {

        // check that the test creates a good template
        Y.Assert.isNotNull(Y.one('.container .active-songs-view li'));

    },

    "afterPlaylistChange: empty playlists": function () {
        this.module.afterPlaylistChange({ newVal: [] });

        Y.Assert.isTrue(this.module.get('nextButton').get('disabled'), 'next disabled');
        Y.Assert.isTrue(this.module.get('playButton').get('disabled'), 'play disabled');
        Y.Assert.isTrue(this.module.get('prevButton').get('disabled'), 'prev disabled');
    },

    "afterPlaylistChange: playlist with one song": function () {
        this.module.set('index', 0);
        this.module.afterPlaylistChange({ newVal: [{}] });

        Y.Assert.isFalse(this.module.get('nextButton').get('disabled'), 'next enabled');
        Y.Assert.isFalse(this.module.get('playButton').get('disabled'), 'play enabled');
        Y.Assert.isTrue(this.module.get('prevButton').get('disabled'), 'prev disabled');
    },

    "onUpdateSelected: (not playing) current song updated: ": function () {
        var mockHistory     = this.module.get('history'),
            song            = { sID: 'foo' },
            box             = Y.one('.active-songs-view');

        // mock history get
        Y.Mock.expect(mockHistory, { method: 'get', args: ['view'] });

        // mock the song node
        box.append(Y.Node.create('<p>').setAttribute('data-u1m-songid', song.sID));

        this.module.onUpdateSelected({ song: song });

        // our song is now "current"
        Y.Assert.isTrue(Y.one('[data-u1m-songid='+song.sID+']').hasClass('current'), 'updated selected');
        // check that the call to history was made
        Y.Mock.verify(mockHistory);

    },

    "updateCurrentTrackRow: ": function () {

        var songsid = 'bar';

        this.module.updateCurrentTrackRow({
            song: { sID: songsid },
            node: this._getSongNode(songsid)
        });

        Y.Assert.isTrue(this._getSongNode(songsid).hasClass('current'), 'current track updated');

    },

    "getTime: test resilience": function () {
        var values  = [-1, 0, 1, 10000, "1000", "0", undefined],
            t       = this.module.getTime;

        Y.each(values, function (val) {
            Y.Assert.isObject(t(val), 'getTime failed');
            Y.Assert.isString(t(val, true), 'getTime failed');
        });
    }

}));

suite.add(new Y.Test.Case({

    name: 'AlbumListView',

    setUp: function () {

        var mockHistory     = Y.Mock();

        Y.Mock.expect(mockHistory, { method: 'after', args: [Y.Mock.Value.String, Y.Mock.Value.Function, Y.Mock.Value.Object] });
        Y.Mock.expect(mockHistory, { method: 'get', args: [Y.Mock.Value.String] });

        setupTemplate();

        this.module = new Y.Music.AlbumListView(Y.merge(widgetConfig, { history: mockHistory }));

    },

    tearDown: function () {
        tearDownTemplate();

        if(this.module) {
            this.module.destroy();
            delete this.module;
        }
    },

    "instantiated ok": function () {
        Y.Assert.isInstanceOf(Y.Music.AlbumListView, this.module);
    },

    "onAlbumClick: history changed": function () {

        var mockHistory     = this.module.get('history'),
            mockTarget      = Y.Node.create('<p><a data-u1m-albumid="fooz"></a></p>');

        Y.Mock.expect(mockHistory, { method: 'add', args: [Y.Mock.Value.Object] });

        this.module.onAlbumClick({
            preventDefault: function () {},
            currentTarget:  mockTarget,
            target:         mockTarget
        });

        Y.Mock.verify(mockHistory);
    }

}));

suite.add(new Y.Test.Case({

    name: 'ArtistListView',

    setUp: function () {

        var mockHistory = Y.Mock();

        Y.Mock.expect(mockHistory, { method: 'on', args: [Y.Mock.Value.String, Y.Mock.Value.Function, Y.Mock.Value.Object]});
        Y.Mock.expect(mockHistory, { method: 'get', args: [Y.Mock.Value.String] });

        setupTemplate();

        this.module = new Y.Music.ArtistListView(Y.merge(widgetConfig, { history: mockHistory }));

    },

    tearDown: function () {
        tearDownTemplate();

        if(this.module) {
            this.module.destroy();
            delete this.module;
        }
    },

    "instantiated ok": function () {
        Y.Assert.isInstanceOf(Y.Music.ArtistListView, this.module);
    },

    "onArtistClick: history changed": function () {

        var mockHistory     = this.module.get('history'),
            mockTarget      = Y.Node.create('<p><a data-u1m-artistid="fooz"></a></p>');

        Y.Mock.expect(mockHistory, { method: 'add', args: [Y.Mock.Value.Object] });

        this.module.onArtistClick({
            preventDefault: function () {},
            currentTarget:  mockTarget
        });

        Y.Mock.verify(mockHistory);
    }

}));

suite.add(new Y.Test.Case({

    name: 'MenuView',

    setUp: function () {

        var mockDatasource  = Y.Mock(),
            mockHistory     = Y.Mock();

        Y.Mock.expect(mockHistory, { method: 'get', args: [Y.Mock.Value.String] });
        Y.Mock.expect(mockDatasource, { method: 'getPlaylists', args: [Y.Mock.Value.Function] });

        setupTemplate();

        this.module = new Y.Music.MenuView(Y.merge(widgetConfig, {
            history:        mockHistory,
            datasource:     mockDatasource
        }));

    },

    tearDown: function () {
        tearDownTemplate();

        if(this.module) {
            this.module.destroy();
            delete this.module;
        }
    },

    "instantiated ok": function () {
        Y.Assert.isInstanceOf(Y.Music.MenuView, this.module);
    },

    "updateActive: class ": function () {

        this.module.updateActive('some-thing');

        Y.Assert.isTrue(Y.one('.some-thing').hasClass('active'));

    },

    "updateActive: node ": function () {

        this.module.updateActive(Y.one('.some-thing'));

        Y.Assert.isTrue(Y.one('.some-thing').hasClass('active'));

    }

}));

suite.add(new Y.Test.Case({

    name: 'SongView',

    _should: {
        ignore: {
            "instantiated ok": true
        }
    },

    setUp: function () {

        var mockHistory     = Y.Mock();

        Y.Mock.expect(mockHistory, { method: 'after', args: [Y.Mock.Value.String, Y.Mock.Value.Function, Y.Mock.Value.Object] });
        Y.Mock.expect(mockHistory, { method: 'get', args: [Y.Mock.Value.String] });

        setupTemplate();

        this.module = new Y.Music.SongView(Y.merge(widgetConfig, { history: mockHistory }));

    },

    tearDown: function () {
        tearDownTemplate();

        if(this.module) {
            this.module.destroy();
            delete this.module;
        }
    },

    "instantiated ok": function () {
        Y.Assert.isInstanceOf(Y.Music.SongView, this.module);
    }


}));

suite.add(new Y.Test.Case({

    name: 'NowPlayingListView',

    setUp: function () {

        var mockHistory     = Y.Mock();
        Y.Mock.expect(mockHistory, { method: 'after', args: [Y.Mock.Value.String, Y.Mock.Value.Function, Y.Mock.Value.Object] });
        Y.Mock.expect(mockHistory, { method: 'get', args: [Y.Mock.Value.String] });

        setupTemplate();

        this.module = new Y.Music.NowPlayingListView(Y.merge(widgetConfig, { history: mockHistory }));

    },

    tearDown: function () {
        tearDownTemplate();

        if(this.module) {
            this.module.destroy();
            delete this.module;
        }
    },

    "instantiated ok": function () {
        Y.Assert.isInstanceOf(Y.Music.NowPlayingListView, this.module);
    }


}));

suite.add(new Y.Test.Case({

    name: 'PlaylistListView',

    setUp: function () {

        var mockHistory     = Y.Mock();
        Y.Mock.expect(mockHistory, { method: 'after', args: [Y.Mock.Value.String, Y.Mock.Value.Function, Y.Mock.Value.Object] });
        Y.Mock.expect(mockHistory, { method: 'get', args: [Y.Mock.Value.String] });

        setupTemplate();

        this.module = new Y.Music.PlaylistListView(Y.merge(widgetConfig, {
            history:        mockHistory,
            events:         { length: false, push: function () {} }
        }));

    },

    tearDown: function () {
        tearDownTemplate();

        if(this.module) {
            this.module.destroy();
            delete this.module;
        }
    },

    "instantiated ok": function () {
        Y.Assert.isInstanceOf(Y.Music.PlaylistListView, this.module);
    }


}));

suite.add(new Y.Test.Case({

    name: 'SongListView',

    setUp: function () {

        var mockHistory     = Y.Mock();
        Y.Mock.expect(mockHistory, { method: 'after', args: [Y.Mock.Value.String, Y.Mock.Value.Function, Y.Mock.Value.Object] });
        Y.Mock.expect(mockHistory, { method: 'get', args: [Y.Mock.Value.String] });

        setupTemplate();

        this.module = new Y.Music.SongListView(Y.merge(widgetConfig, {
            history:        mockHistory,
            events:         { length: false, push: function () {} }
        }));

    },

    tearDown: function () {
        tearDownTemplate();

        if(this.module) {
            this.module.destroy();
            delete this.module;
        }
    },

    "instantiated ok": function () {
        Y.Assert.isInstanceOf(Y.Music.SongListView, this.module);
    }


}));

suite.add(new Y.Test.Case({

    name: 'MySongListView'


}));

suite.add(new Y.Test.Case({

    name: 'PlaylistGridView',

    setUp: function () {

        var mockHistory     = Y.Mock();
        Y.Mock.expect(mockHistory, { method: 'after', args: [Y.Mock.Value.String, Y.Mock.Value.Function, Y.Mock.Value.Object] });
        Y.Mock.expect(mockHistory, { method: 'get', args: [Y.Mock.Value.String] });

        setupTemplate();

        this.module = new Y.Music.PlaylistGridView(Y.merge(widgetConfig, {
            history:        mockHistory,
            events:         { length: false, push: function () {} }
        }));

    },

    tearDown: function () {
        tearDownTemplate();

        if(this.module) {
            this.module.destroy();
            delete this.module;
        }
    },

    "instantiated ok": function () {
        Y.Assert.isInstanceOf(Y.Music.PlaylistGridView, this.module);
    }


}));


Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: ['one-music-views', 'template']
});
