    var Pages = Y.Base.create('pages', Y.View, [], {

        initializer: function(config) {
            var numberOfPages = this.get('numberOfPages'),
                context = {
                    page: this.get('page'),
                    numberOfPages: numberOfPages,
                    startItem: this.get('startItem'),
                    endItem: this.get('endItem'),
                    totalItems: this.get('totalItems')
                },
                curPage,
                pageList,
                pagination,
                prevNode,
                prevPage,
                nextPage,
                firstPage,
                lastPage,
                newContext,
                chunk,
                template,
                i, j,

                // should be '?page={page}' or something of the sort
                URLTemplate = config.URLTemplate || '#',
                escapeHtml = function(k,v){
                    return Y.Escape.html(v);
                };




            pageList = this.getPageList();

            // create url for previous <
            context.prevUrl = Y.substitute(URLTemplate, {
                page: this.get('prevPage')
            });
            // create url for next >
            context.nextUrl = Y.substitute(URLTemplate, {
                page: this.get('nextPage')
            });

            pagination = Y.Node.create(Y.substitute(this.templates.body, context, escapeHtml));

            config.srcNode.setContent(pagination);
            prevNode = config.srcNode.one('.prev');

            prevPage = config.srcNode.one('.prev-page');
            nextPage = config.srcNode.one('.next-page');

            // work backwards through the pageList array
            // to aid adding nodes in the correct order.
            for (i=pageList.length, j=1; i >= j; i--) {

                newContext = Y.clone(context);
                curPage = newContext.page = pageList[i-1];

                template = null;
                if (curPage === context.page) {
                    template = this.templates.partialNoLink;
                } else {
                    template = this.templates.partialLink;

                    // normal page item, so create a url
                    newContext.url = Y.substitute(URLTemplate, {
                        page: curPage
                    });
                }
                chunk = Y.Node.create(Y.substitute(template, newContext, escapeHtml));
                if (i === 1) {
                    chunk.one('.num').addClass('first');
                }
                prevNode.insert(chunk, 'after');
            }

            if (Y.Array.indexOf(pageList, numberOfPages) === -1) {

                // create url for endlink
                newContext.url = Y.substitute(URLTemplate, {
                    page: numberOfPages
                });
                context.url = Y.substitute(URLTemplate, {
                    page: numberOfPages
                });

                template = this.templates.endLink;

                chunk = Y.Node.create(Y.substitute(template, context, escapeHtml));
                nextPage.get('parentNode').insert(chunk, 'before');
            }

            if (Y.Array.indexOf(pageList, 1) === -1) {
                // create url for startlink
                chunk = Y.Node.create(Y.substitute(this.templates.startLink, {url: Y.substitute(URLTemplate, { page: 1 }) }, escapeHtml));
                prevPage.get('parentNode').insert(chunk, 'after');
            }

            if (context.page === 1) {
                if (prevPage) {
                    prevPage.get('parentNode').setContent('<span>' + prevPage.getContent() + '</span>');
                }
                firstPage = config.srcNode.one('.first-page');
                if (firstPage) {
                    firstPage.get('parentNode').setContent(firstPage.getContent());
                }
            }

            if (context.page === context.numberOfPages) {
                if (nextPage) {
                    nextPage.get('parentNode').setContent('<span>' + nextPage.getContent() + '</span>');
                }
                lastPage = config.srcNode.one('.last-page');
                if (lastPage){
                    lastPage.get('parentNode').setContent(lastPage.getContent());
                }
            }
        },

        /**
        So, the numAdjacentLinks is crucial to understanding what comes out of this function

        Scenario A:
            currentPage = 5
            totalPages = 10
            numberOfAdjacentLinks = 2

            getPageList() would then return [ 3, 4, 5, 6, 7]
            with currentpage in the middle of two links

        Scenario B
            currentPage = 10
            totalPages = 10
            numberOfAdjacentLinks = 2

            getPageList() would then return [ 8, 9, 10 ]

        @return {Array} range The current page in the middle of n adjacent links
        */
        getPageList: function() {

            var page = this.get('page'),
                numAdjacentLinks = this.get('numAdjacentLinks'),
                numberOfPages = this.get('numberOfPages'),
                startPage = (page - numAdjacentLinks),
                endPage = (page + numAdjacentLinks);

            // if there are few pages following or adjacent links,
            // include them
            if (endPage < (2 * numAdjacentLinks) + 1) {
                endPage = (2 * numAdjacentLinks) + 1;
            }

            if (startPage > (numberOfPages - (2 * numAdjacentLinks))) {
                startPage = (numberOfPages - (2 * numAdjacentLinks));
            }

            // if startPage should be less than 1, set it to 1
            startPage = startPage >= 1 ? startPage : 1;

            // if endPage is larger than the number of pages, set it to the number of pages
            endPage = (endPage < numberOfPages) ? endPage : numberOfPages;

            // return a range from startPage to endPage + 1
            return this.range(startPage, (endPage + 1));
        },

        range: function(start, end) {
            var list = [], i;
            for (i = start; i < end; i++){
                list.push(i);
            }
            return list;
        },

        hasNext: function() {
            return (this.get('page') + 1) <= this.get('numberOfPages');
        },

        hasPrev: function() {
            return (this.get('page') - 1) >= 1;
        },

        templates: {
            body: [
                '<ul class="pager clearfix">',
                '<li class="page-details">Photos {startItem}–{endItem} of {totalItems}</li>',
                '<li class="prev"><a title="Previous page" class="prev-page" href="{prevUrl}">&lsaquo;</a></li>',
                '<li class="next"><a title="Next page" class="next-page" href="{nextUrl}">&rsaquo;</a></li>',
                '</ul>'].join(''),
            partialLink: '<li><a class="num" href="{url}">{page}</a></li>',
            partialNoLink: '<li><span class="num current">{page}</span></li>',
            startLink: [
                '<li class="start"><a title="First page" class="first-page" href="{url}">1</a></li>',
                '<li><span>&hellip;</span></li>'
            ].join(''),
            endLink: [
                '<li><span>&hellip;</span></li>',
                '<li class="end"><a title="Next page" class="last-page" data-lastpage="{pages}" href="{url}">{numberOfPages}</a></li>'
            ].join('')
        }
    }, {
        ATTRS: {

            /**
            The index of the first item displayed

            @attribute startItem
            @type Number
            */
            startItem: {
                value: null,
                getter: function (val) {
                    var startItem,
                        totalItems;

                    if(!val) {
                        startItem = (this.get('page') - 1 ) * this.get('pageSize') + 1;
                        totalItems = this.get('totalItems');

                        if(startItem < 1) {
                            startItem = 1;
                        }
                        val = startItem;
                    }

                    return val;
                }
            },

            /**
            The index of the last item displayed

            @attribute endItem
            @type Number
            */
            endItem: {
                value: null,
                getter: function (val) {
                    var endItem,
                    totalItems;

                    if(!val) {
                        endItem = this.get('page') * this.get('pageSize');
                        totalItems = this.get('totalItems');

                        if(endItem > totalItems) {
                            endItem = totalItems;
                        }
                        val = endItem;
                    }

                    return val;
                }
            },

            /**
            Amount of pager links to be rendered next to current page

            @attribute numAdjacentLinks
            @default {Number} 2
            @type Number
            */
            numAdjacentLinks: { value: 2 },

            /**
            Number of pages, total

            @attribute numberOfPages
            @type Number
            */
            numberOfPages: {
                value: null,
                getter: function (val) {
                    if(!val) {
                        val = Math.ceil(this.get('totalItems') / this.get('pageSize'));
                    }
                    return val;
                }
            },

            /**
            The current page index

            @attribute page
            @type Number
            */
            page: {
                value: null,
                setter: function (val) {
                    var n = Y.Lang.isNumber(val) ? val : parseInt(val, 10),
                        numberOfPages = this.get('numberOfPages');

                    // if the page is way high, set it to 1
                    // this mimics the server
                    if(n > numberOfPages) {
                        n = 1;
                    }

                    // if page is 0 or less
                    if(n < 1) {
                        n = 1;
                    }

                    return n;
                }
            },


            /**
            Number of items available, in total

            @attribute totalItems
            @type Number
            */
            totalItems: {
                value: null,
                setter: function (val) {
                    var n = Y.Lang.isNumber(val) ? val : parseInt(val, 10);
                    return n;
                }
            },

            /**
            Number of items displayed on each page

            @attribute pageSize
            @type Number
            */
            pageSize: {
                value: null,
                setter: function (val) {
                    var n = Y.Lang.isNumber(val) ? val : parseInt(val, 10);
                    return n;
                }
            },

            /**
            Next page index

            @attribute nextPage
            @type Number
            */
            nextPage: {
                value: null,
                getter: function (val) {
                    var nextPage,
                        numberOfPages;

                    if(!val) {
                        nextPage = this.get('page') + 1;
                        numberOfPages = this.get('numberOfPages');

                        if(nextPage > numberOfPages) {
                            nextPage = numberOfPages;
                        }

                        val = nextPage;

                    }

                    return val;
                }
            },

            /**
            Previous page index

            @attribute prevPage
            @type Number
            */
            prevPage: {
                value: null,
                getter: function (val) {
                    var prevPage;

                    if(!val) {
                        prevPage = this.get('page') - 1;

                        if(prevPage < 1) {
                            prevPage = 1;
                        }
                        val = prevPage;
                    }

                    return val;
                }
            }
        }
    });
    Y.Pages = Pages;
