YUI.add('one-pages-test', function (Y) {

var Assert = Y.Assert,
    suite;

suite = new Y.Test.Suite('Pages');

/*
Pages have barely four testable units and one HUGE initializer

TODO: break the initializer down to smaller units and test them
*/

suite.add(new Y.Test.Case({

    setUp: function () {
        this.app = Y.Pages.prototype;

        // fake the Pages module and let the tests override variables
        this.mockThis = {
            
            numberOfPages: 10,
            page: 5,
            numAdjacentLinks: 2,

            get: function (what) {
                var ret;

                // numberOfPages
                if(what === 'numberOfPages') {
                    ret = this.numberOfPages;
                }

                // page
                else if (what === 'page') {
                    ret = this.page;
                }

                // numAdjacentLinks
                else if (what === 'numAdjacentLinks') {
                    ret = this.numAdjacentLinks;
                }

                return ret;
            }
        };
    },

    tearDown: function () {
        delete this.app;
        delete this.mockThis;
    },

    "getPageList(): get pagelist with normal, in the middle values": function () {
        // this datavalidation is based on stuff we put into this.mockThis in the setup
        // so a bit brittle, be aware
        var pageList = [3,4,5,6,7];

        // 'import' function range
        this.mockThis.range = this.app.range;

        Y.ArrayAssert.itemsAreSame(pageList, this.app.getPageList.call(this.mockThis), 'getPageList result yielded ALL required items');
        
    },

    "getPageList(): get pagelist where currentpage is in the lower end with just one adjacent link": function () {
        var targetPageList = [1,2,3],
            actualPageList;

        // since the amount of adjacent links + page is less than (2 * adjacent links + 1), the end of the list will be (2 * adjacent links + 1), which is 3
        this.mockThis = Y.merge(this.mockThis, {
            page: 1,
            numberOfPages: 10,
            numAdjacentLinks: 1
        });

        // 'import' function range
        this.mockThis.range = this.app.range;

        actualPageList = this.app.getPageList.call(this.mockThis);

        Y.ArrayAssert.itemsAreSame(targetPageList, actualPageList, 'got this list ' 
            + actualPageList.toString() 
            + ' which is actually correct ref our target: '
            + targetPageList.toString()
            + '. For more about this read more in this test or in the getPageList docs.');
        
    },

    "getPageList(): get pagelist where currentpage is in the lower end with three adjacent links": function () {
        var targetPageList = [1,2,3,4,5,6,7],
            actualPageList;

        // since the amount of adjacent links + page is less than (2 * adjacent links + 1), the end of the list will be (2 * adjacent links + 1), which is 7
        this.mockThis = Y.merge(this.mockThis, {
            page: 1,
            numberOfPages: 10,
            numAdjacentLinks: 3
        });

        // 'import' function range
        this.mockThis.range = this.app.range;

        actualPageList = this.app.getPageList.call(this.mockThis);

        Y.ArrayAssert.itemsAreSame(targetPageList, actualPageList, 'got this list '
            + actualPageList.toString()
            + ' which is actually correct ref our target: '
            + targetPageList.toString()
            + '. For more about this read more in this test or in the getPageList docs.');
        
    },

    "getPageList(): get pagelist where currentpage is in the higher end with just one adjacent link": function () {
        var targetPageList = [8,9,10],
            actualPageList;

        // the logic here is:
        // if page is larger than numberOfPages - ( 2 * adjacent links ), which is 10 > 10 - ( 2 * 1 ), which is 10 > 8,
        // which is true, startPage gets set to 8
        this.mockThis = Y.merge(this.mockThis, {
            page: 10,
            numberOfPages: 10,
            numAdjacentLinks: 1
        });

        // 'import' function range
        this.mockThis.range = this.app.range;

        actualPageList = this.app.getPageList.call(this.mockThis);

        Y.ArrayAssert.itemsAreSame(targetPageList, actualPageList, 'got this list '
            + actualPageList.toString()
            + ' which is actually correct ref our target: '
            + targetPageList.toString()
            + '. For more about this read more in this test or in the getPageList docs.');
        
    },

    "getPageList(): get pagelist on huge list": function () {
        var targetPageList = [298, 299, 300, 301, 302],
            actualPageList;

        this.mockThis = Y.merge(this.mockThis, {
            page: 300,
            numberOfPages: 1326,
            numAdjacentLinks: 2
        });

        // 'import' function range
        this.mockThis.range = this.app.range;

        actualPageList = this.app.getPageList.call(this.mockThis);

        Y.ArrayAssert.itemsAreSame(targetPageList, actualPageList, 'got correct list');
    },

    "getPageList(): get pagelist at the end of monster list": function () {
        var targetPageList = [16994, 16995, 16996, 16997, 16998],
            actualPageList;

        this.mockThis = Y.merge(this.mockThis, {
            page: 16996,
            numberOfPages: 17000,
            numAdjacentLinks: 2
        });

        // 'import' function range
        this.mockThis.range = this.app.range;

        actualPageList = this.app.getPageList.call(this.mockThis);

        Y.ArrayAssert.itemsAreSame(targetPageList, actualPageList, 'got correct list');
    },


    "range(): create a range with normal values": function () {
        var size = 100,
            range = this.app.range(0, size);
        Assert.areSame(range.length, size, 'The length of returned range was ' + size);
    },




    "hasNext(): have next item in list of normal values": function () {
        Assert.isTrue(this.app.hasNext.call(this.mockThis), 'normal list in the middle has next');
    },

    "hasNext(): have next item in small list currently at the beginning": function () {
        this.mockThis = Y.merge(this.mockThis, {
            page: 1,
            numberOfPages: 2
        });
        Assert.isTrue(this.app.hasNext.call(this.mockThis), 'small list at the beginning has next');
    },

    "hasNext(): no next list item in list with only one item": function () {
        this.mockThis = Y.merge(this.mockThis, {
            page: 1,
            numberOfPages: 1
        });
        Assert.isFalse(this.app.hasNext.call(this.mockThis), 'no next item in list');
    },




    "hasPrev(): have prev item in list of normal values": function () {
        Assert.isTrue(this.app.hasPrev.call(this.mockThis), 'normal list in the middle has prev');
    },

    "hasPrev(): have prev item in small list currently at the end": function () {
        this.mockThis = Y.merge(this.mockThis, {
            page: 2,
            numberOfPages: 2
        });
        Assert.isTrue(this.app.hasPrev.call(this.mockThis), 'small list at the end has prev');
    },

    "hasPrev(): no prev list item in list with only one item": function () {
        this.mockThis = Y.merge(this.mockThis, {
            page: 1,
            numberOfPages: 1
        });
        Assert.isFalse(this.app.hasPrev.call(this.mockThis), 'no prev item in list');
    },

    "getPageList(): reproducing behaviour where last digit in a two or more page number would be appended to all page items (#1085091)": function () {
        var pagelist,
            targetPageList = [11,12,13,14,15];

        this.mockThis = Y.merge(this.mockThis, {
            page: 13,
            numberOfPages: 40
        });

        // 'import' function range
        this.mockThis.range = this.app.range;

        pagelist = this.app.getPageList.call(this.mockThis);

        Y.ArrayAssert.itemsAreSame(targetPageList, pagelist, 'got correct list');
    }

}));

/*
suite.add(new Y.Test.Case({

    "use ok": function () {
        
        // fails miserably
        //Assert.isObject(new Y.Pages());

    }

}));
*/

Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: ['one-pages', 'test']
});
