String.prototype.repeat = function (nTimes) {
  var sDiff = "", sBase2 = nTimes > 0 ? this.valueOf() : "";
  for (var nMask = nTimes; nMask > 1; nMask >>= 1) {
    if (nMask & 1) { sDiff += sBase2; }
    sBase2 += sBase2;
  }
  return sBase2 + sDiff;
};

YUI.add('one-password-meter-test', function (Y) {

var Assert = Y.Assert,
    suite;


suite = new Y.Test.Suite('one-password-meter');

suite.add(new Y.Test.Case({

    name: '',

    setUp : function () {

        this.meter = new Y.U1.Widgets.PasswordMeter({
            input: '.pwd',
            inputConfirm: '.confirmPwd'
        });
        this.meter.render();

        this.input = Y.one('.pwd');
        this.inputConfirm = Y.one('.confirmPwd');
    },

    tearDown: function () {
        this.meter.destroy();
        this.input.set('value', '');
        this.inputConfirm.set('value', '');
    },

    "short passwords": function () {
        var s = 's',
            i = 0,
            j = this.meter.get('minimumLength');

        for(; i<j; i++) {

            this.input.set('value', s.repeat(i));
            this.meter._setStrength();

            Assert.areSame(this.meter.get('strength'), 'short');
        }
    },

    "sums: all lower case a x 8 is 0": function () {
        Assert.areSame(0, this.meter.get('passwordChecker')("aaaaaaaa"));
    },

    "sums: all lower case a x 1000 gets a point for length": function () {
        Assert.areSame(1, this.meter.get('passwordChecker')("a".repeat(1000)));
    },

    "sums: aaaaaaaA, one point for lower/upper case combo": function () {
        Assert.areSame(1, this.meter.get('passwordChecker')("aaaaaaaA"));
    },

    "sums: aaaaaaA1, two points for combo and number": function () {
        Assert.areSame(2, this.meter.get('passwordChecker')("aaaaaaA1"));
    },

    "sums: aaaaa_A1, three points for combo, number and symbol": function () {
        Assert.areSame(3, this.meter.get('passwordChecker')("aaaaa_A1"));
    },

    "sums: aaaaa_A1zzzzzzzzzzzz, four points for combo, number, symbol and length": function () {
        Assert.areSame(4, this.meter.get('passwordChecker')("aaaaa_A1zzzzzzzzzzzz"));
    },

    "no inputs were harmed in the making of this widget": function () {
        this.meter.destroy();
        Assert.isNotNull(Y.one('.pwd'));
    },

    // tests that we have slideup/slidedown
    // when keyup happens in a populated pwd field
    "input events": function () {
        this.input.set('value', 'fo');
        this.input.simulate('keyup');

        // not enough chars to show the indicator
        Assert.areSame('0px', this.meter._indicatorNode.getComputedStyle('height'));

        this.input.set('value', 'AHSHDASwde838d3');
        this.input.simulate('keyup');

        // the animation takes a bit of time, wait for
        // the indicator to slideDown
        this.wait(function () {
            Assert.isTrue(parseInt(this.meter._indicatorNode.getComputedStyle('height'), 10) > 0);
        }, 300);

    },

    "only one strength class": function () {
        var pwds = ['0239__sss__w929A   AA', 'aaaaaaaa', 'aaaaaaaa000000', 'fooo', '', '99999999999', 'aaaaaa', 'aaaaaaa', 'aaaaaaaa'];

        Y.Array.each(pwds, function (p) {
            this.input.set('value', p);
            this.input.simulate('keyup');
            Assert.areSame(1, this.meter.get('boundingBox').getAttribute('class').match(/strength/g).length);
        }, this);
    },

    "asymmetric password fields": function () {

        this.input.set('value', 'AHSHDASwde838d3');
        this.input.simulate('keyup');

        this.inputConfirm.set('value', 'notsimilar');
        this.inputConfirm.simulate('blur');

        // the animation takes a bit of time, wait for
        // the indicator to slideDown
        this.wait(function () {
            Assert.isNotNull(Y.one('.' + this.meter.getClassName('strength-asymmetric')));
        }, 300);

    },

    "strength can be anything": function () {
        this.meter.set('strength', 999);
    },

    "strength can be strings": function () {
        this.meter.set('strength', 'foo');
    }


}));


Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: ['one-password-meter', 'test', 'node-event-simulate']
});
