var PhotosListView = Y.Base.create('photosListView', Y.View, [], {

    albumImagesGroupConfig: {
        name: "photosImgGroup",
        foldDistance: 30,
        foldCheckDelay: 200
    },

    initializer: function() {

        this.albumImagesGroup = new Y.U1.ImgLoadGroup(this.albumImagesGroupConfig);

        // find and register images given supplied by server
        this.get('container').all('img').each(function (image) {
            this.registerImage(image, image.getData('u1src'));
        }, this);

    },

    registerImage: function (image, src) {
        var id = Y.Event.generateId(image);
        image.set('id', id);

        this.albumImagesGroup.registerImage({
            domId: id,
            srcUrl: src
        });

        image.on('load', function () {
            image.transition({
                opacity: 1
            }, function () {
                this.removeClass('pending-fadein');
            });
        });
    },

    template: Y.one('#photo-listing').getContent(),

    render: function (app) {
        // destroy/create a imgloadgroup
        this.albumImagesGroup.destroy();
        this.albumImagesGroup = new Y.U1.ImgLoadGroup(this.albumImagesGroupConfig);

        var photos = this.get('photoList'),
            self = this,
            container = this.get('container'),
            params = app.getQueryString();

        // remove nodes and remove events
        container.all('li').remove().destroy(true);

        if(params) {
            params = '?' + params;
        }

        Y.log('photos size' + photos.size());

        photos.each(function (photo) {

            // create node
            var node = Y.Node.create(
                    Y.substitute(
                        self.template,
                        Y.merge(photo, { params: params }),
                        function(k,v){return Y.Escape.html(v);}
                    )
                ),

                thumbnail_url = photo.thumbnail_url + '/400x400/?constrain=1',
                public_thumbnail_url = photo.public_thumb_base_url + '/400x400/' + photo.name + '?constrain=1',
                url,
                imgNode = node.one('img');

            url = photos.get('public_list') ? public_thumbnail_url : thumbnail_url;

            // register in imageload
            self.registerImage(imgNode, url);

            // insert node
            container.appendChild(node);

        });

    }

}, {
    ATTRS: {
        container: {
            value: '.photo-list'
        }
    }
});

Y.namespace('Photos').PhotosListView = PhotosListView;
