YUI.add('one-photos-common-test', function (Y) {

var Assert = Y.Assert,
    suite;

suite = new Y.Test.Suite('Photos Common');

suite.add(new Y.Test.Case({

    name: 'PhotosCommon',

    setUp : function () {

        this.app = Y.Photos.PhotosCommon.prototype;
        this.mockThis = {
            addressBar: '',
            navigate: function (url) {
                this.addressBar = url;
            }
        };

    },

    tearDown: function () {
        delete this.mockThis;
        delete this.app;
    },

    "cleanQueryObjectParameters(): Cleaning a messy query": function() {
        var queryObject = {
            'keep_this': 'foo',
            'remove_this': 'bar'
        },
            cleanQueryObject;

        this.mockThis.queryParameters = ['keep_this'];

        cleanQueryObject = this.app.cleanQueryObjectParameters.call(this.mockThis, queryObject);

        Y.assert(cleanQueryObject.remove_this === undefined, 'Removed remove_this');
        Y.assert(cleanQueryObject.keep_this !== undefined, 'Kept keep_this');
    },

    "getQueryString(): get query string": function () {
        var string = 'foo=bar&n=1000%20&13=_3';
        Y.config.win = {
            location: {
                search: '?' + string
            }
        };
        Assert.areSame(string, this.app.getQueryString(), 'Mock location.search same as mock string minus ?');
    },

    "getHashQueryString(): get query string from hash in an non-html5 pushstate browser with weird path": function () {
        var string = 'foo=bar&n=1000%20&13=_3';
        Y.config.win = { location: {
            href: 'http://example.org?something=else#/some/weird/path?' + string,
            hash: '#/some/weird/path?' + string
        }};
        Assert.areSame(string, this.app.getHashQueryString(), 'Mock location.href hash string parameters same as mock string');
    },

    "getHashQueryString(): get query string from hash in an non-html5 pushstate browser without path": function () {
        var string = 'foo=bar&n=1000%20&13=_3';

        Y.config.win = { location: {
            href: 'http://example.org?something=else#?' + string,
            hash: '#?' + string
        }};

        Assert.areSame(string, this.app.getHashQueryString(), 'Mock location.href hash string parameters same as mock string');
    },

    "getQueryStringAsObject(): get query string as object without parameters": function () {
        var string = 'foo=bar&n=1000%20&13=_3',
            keys = [
                'foo',
                'n',
                '13'
            ];
        Y.config.win = {
            location: {
                search: '?' + string
            }
        };

        // required function
        this.mockThis.getHTML5QueryString = this.app.getHTML5QueryString;
        this.mockThis.getQueryString = this.app.getQueryString;

        Y.ObjectAssert.ownsKeys(keys, this.app.getQueryStringAsObject.call(this.mockThis), 'Found all keys');
    },

    "getQueryStringAsObject(): get query string as object with string parameter": function () {
        var string = 'foo=bar&n=1000%20&13=_3',
            keys = [
                'foo',
                'n',
                '13'
            ];

        Y.ObjectAssert.ownsKeys(keys, this.app.getQueryStringAsObject.call(this.mockThis, string), 'Found all keys');
    },

    "getQueryStringAsObject(): get query string as object with empty string parameter": function () {
        var string = '';

        // required function
        this.mockThis.getQueryString = this.app.getQueryString;

        Y.ObjectAssert.ownsNoKeys(this.app.getQueryStringAsObject.call(this.mockThis, string), 'Got empty object');
    },

    "updateQueryObjectParameter(): update queryObject": function () {
        var queryObject = {
            foo: 'bar',
            sweets: 'choc chips',
            this_test_sucks: 'yes'
        };

        Assert.areSame('no', this.app.updateQueryObjectParameter(queryObject, 'this_test_sucks', 'no').this_test_sucks, 'Test no longer sucks');
    },

    "updateQueryObjectParameter(): update queryObject with new key": function () {
        var queryObject = {
            foo: 'bar',
            sweets: 'choc chips',
            this_test_sucks: 'yes'
        };

        Assert.areSame('weee', this.app.updateQueryObjectParameter(queryObject, 'new_key', 'weee').new_key, 'Non-existing key now on object');
    },

    "updateQueryObjectParameter(): update queryObject with empty key": function () {
        var queryObject = {
            foo: 'bar',
            sweets: 'choc chips',
            this_test_sucks: 'yes'
        },
        functionThatThrowsError = Y.bind(this.app.updateQueryObjectParameter, this.mockThis, queryObject, '', 'foo');

        Assert.throwsError('updateQueryObjectParameter: Cannot update queryObject with falsy parameter', functionThatThrowsError);
    },

    "sanitizeQueryParameters(): removing all queryparameters because they are default": function () {
        var queryObject = {
            page: 1,
            pagesize: 18,
            folder_filter: 'my'
        };

        this.mockThis.defaultQueryParameters = {
            page: 1,
            pagesize: 18,
            folder_filter: 'my'
        };

        Y.ObjectAssert.ownsNoKeys(this.app.sanitizeQueryParameters.call(this.mockThis, queryObject), 'Got empty object');
    },

    "sanitizeQueryParameters(): not removing the one parameter that is not default": function () {
        var queryObject = {
            page: 100000000000000000,
            pagesize: 18,
            folder_filter: 'my',
            dummy: 'foo'
        };

        this.mockThis.defaultQueryParameters = {
            page: 1,
            pagesize: 18,
            folder_filter: 'my'
        };

        Y.ObjectAssert.ownsKey('page', this.app.sanitizeQueryParameters.call(this.mockThis, queryObject), 'Got the page key back.');
    },

    "sanitizeQueryParameters(): keeping parameter by second argument to function": function () {
        var queryObject = {
            page: 1,
            pagesize: 18,
            folder_filter: 'my'
        },
            keep = 'page';

        this.mockThis.defaultQueryParameters = {
            page: 1,
            pagesize: 18,
            folder_filter: 'my'
        };

        Y.ObjectAssert.ownsKey(keep, this.app.sanitizeQueryParameters.call(this.mockThis, queryObject, keep), 'Got '+keep+' key back even though it was default.');
    },

    "sanitizeQueryParameters(): keep all parameters": function () {
        var queryObject = {
            page: 1,
            pagesize: 18,
            folder_filter: 'my',
            foo: 'bar'
        },
            keep = [
                'page',
                'pagesize',
                'folder_filter'
            ];

        this.mockThis.defaultQueryParameters = {
            page: 1,
            pagesize: 18,
            folder_filter: 'my'
        };

        Y.ObjectAssert.ownsKeys(keep, this.app.sanitizeQueryParameters.call(this.mockThis, queryObject, keep), 'Got all keys to keep back even though they all were default.');
    }


}));

suite.add(new Y.Test.Case({

    "use ok": function () {
        Assert.isObject(new Y.Photos.PhotosCommon());
    }

}));

Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: ['one-photos-common', 'test']
});
