YUI.add('one-photos-gallery-app-test', function (Y) {

var Assert = Y.Assert,
    suite,
    html5 = Y.Router.html5,
    win = Y.config.win,
    originalURL = (win && win.location.toString()) || '',
    resetURL = function () {
        if (html5) {
            win && win.history.replaceState(null, null, originalURL);
        } else {
            win && (win.location.hash = '');
        }
    };

suite = new Y.Test.Suite('Photos Gallery');

suite.add(new Y.Test.Case({

    name: 'PhotosGalleryAppTests',

    setUp : function () {

        var photoList;

        resetURL();

        // create app, but no routing
        this.app = new Y.Photos.GalleryApp({
            photoList: new Y.Photos.PhotoList({
                items: Y.Photos.TestData.photos,
                path: Y.Photos.TestData.path
            }),
            albumURLScheme: '',
            galleryURLScheme: ''
        });

        photoList = this.app.get('photoList');

        // something for getCurrentPhotoNode
        this.app.get('activeView')
            .get('container')
            .append('<div class="photo"><img data-nodekey="' + photoList.getPhoto('Cars.TIFF').nodekey + '"></div>');

        Y.Photos.TestData.resetPhotoList(photoList);
        this.app.prefetchPhotos = [];
    },

    tearDown: function () {
        this.app && this.app.destroy({remove: true});
        resetURL();
    },

    "getPhotoURL(): no params": function() {

        var createdUrl = this.app.getPhotoURL(Y.Photos.TestData.photos[0]);
            targetUrl = '/photos/gallery/i897aysdasiduih';

        Assert.areSame(targetUrl, createdUrl);
    },

    "getPhotoURL(): with assorted params": function() {
        var createdUrl,
            params = 'photo_order=desc&foo=bar',
            targetUrl = '/photos/gallery/i897aysdasiduih?'+params;


        this.app.save(Y.config.win.location.href + '?' + params);

        this.wait(function () {
            createdUrl = this.app.getPhotoURL(Y.Photos.TestData.photos[0]);
            Assert.areSame(targetUrl, createdUrl, 'Good URL + params');
        }, 200);
    },
    "getPhotoURL(): with extraParams": function() {

        var createdUrl,
            extraParams = {
                foo: 'bar',
                slideshow: 1
            },
            targetUrl = '/photos/gallery/i897aysdasiduih?foo=bar&slideshow=1';

        createdUrl = this.app.getPhotoURL(Y.Photos.TestData.photos[0], extraParams);
        Assert.areSame(targetUrl, createdUrl, 'Good URL + params');
    },

    "getPhotoURL(): with assorted params + extraParams": function() {
        var createdUrl,
            params = 'photo_order=desc&baz=bzr',
            extraParams = {
                foo: 'bar',
                slideshow: 1
            },
            targetUrl = '/photos/gallery/i897aysdasiduih?photo_order=desc&baz=bzr&foo=bar&slideshow=1';

        this.app.save(Y.config.win.location.href + '?' + params);

        this.wait(function () {
            createdUrl = this.app.getPhotoURL(Y.Photos.TestData.photos[0], extraParams);
            Assert.areSame(targetUrl, createdUrl, 'Good URL with params and extraParams');
        }, 200);

    },

    "getPreviousPhoto(): gets previous photo": function () {
        Y.ObjectAssert.areEqual(this.app.get('photoList').getPhoto('Badvark.jpg'), this.app.getPreviousPhoto());
    },

    "getNextPhoto(): gets next photo": function () {
        // the sort is case sensitive
        Y.ObjectAssert.areEqual(this.app.get('photoList').getPhoto('Freecell.exe'), this.app.getNextPhoto());
    },

    "prefetch(): with 0 as arg": function () {
        this.app.prefetch(0);
        Assert.areSame(3, this.app.prefetchPhotos.length, 'Prefetched three images');
    },

    "prefetch(): n and amount argument": function () {
        this.app.prefetch(0, 100000);
        Assert.isTrue(this.app.prefetchPhotos.length > 3, 'Prefetched more than three images');
    },

    "prefetch(): n is 2": function () {
        this.app.prefetch(0, 2);
        Assert.areSame(2, this.app.prefetchPhotos.length, 'Prefetched two images');
    },


    "prefetch(): forbibly request 10 prefetched images": function () {
        this.app.prefetch(0, 90, 90);
        Assert.areSame(0, this.app.prefetchPhotos.length, 'Prefetched 0 of 90 images');
    },

    "prefetch(): get all images available": function () {
        var n = this.app.get('photoList').size() - 1;
        this.app.prefetch(0, n, n);
        Assert.areSame(n, this.app.prefetchPhotos.length, 'Prefetched ' + n + ' images');
    },

    "prefetch(): no uneccessary prefetching": function () {
        var n = this.app.get('photoList').size() - 1;
        this.app.prefetch(0, n, n);
        this.app.prefetch(0, n, n);
        Assert.areSame(n, this.app.prefetchPhotos.length, 'Prefetched ' + n + ' images');
    }

}));


Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: ['one-photos-gallery-app', 'one-photos-test-shared', 'test']
});
