YUI.add('one-photos-photo-list-model-test', function (Y) {

var Assert = Y.Assert,
    suite;

suite = new Y.Test.Suite('Photo List');

suite.add(new Y.Test.Case({

    name: 'PhotoListTests',

    setUp : function () {

        this.photos = Y.Photos.TestData.photos;

        this.modelList = new Y.Photos.PhotoList({
            items: this.photos,
            photo_order: 'last_modified-asc'
        });

    },

    tearDown: function () {
        this.modelList.destroy();
        delete this.modelList;
    },

    "getPhoto: using id": function () {
        var targetKey = '7493802horsesLOL34839',
            p = this.modelList.getPhoto(targetKey);

        Assert.areSame(p.nodekey, targetKey);
    },

    "getPhoto: using name": function () {
        var targetName = 'cats2.gif',
            p = this.modelList.getPhoto(targetName);

        Assert.areSame(p.name, targetName);
    },

    "getPhoto: using index": function () {
        var targetIndex = 0,
            p = this.modelList.getPhoto(targetIndex);

        Assert.isTrue(!!p.name);
    },

    "getPhoto: non-existing photo": function () {
        var p = this.modelList.getPhoto('NAN');
        Assert.areSame(null, p);
    },

    "multisort (modified asc, name asc)": function () {

        this.modelList.photoSort({
            silent: true
        });

        // so, Aadvark.jpg and Badvark.jpg share the same modified date
        // multisort should then order Aadvark before Badvark
        Assert.areSame('Aadvark.jpg', this.modelList.toArray()[0].name);
        Assert.areSame('Badvark.jpg', this.modelList.toArray()[1].name);
    },

    "multisort (modified desc, name asc)": function () {
        this.modelList.set('photo_order', 'last_modified-desc');
        this.modelList.photoSort({
            silent: true
        });

        Assert.areSame('Aadvark.jpg', this.modelList.toArray()[6].name);
        Assert.areSame('Badvark.jpg', this.modelList.toArray()[7].name);
    },


    "parsing json": function () {

        var json = {
            "results": {
                "cover_photo": "cover.jpg",
                "display_name": "test album",
                "last_modified": 123,
                "owner": "test master",
                "images": [
                    "image1",
                    "image2"
                ]
            }
        };

        // parse function returns json images
        Assert.areSame(json.results.images, this.modelList.parse(json));

        Assert.areSame(json.results.cover_photo, this.modelList.get('cover_photo'));
    }


}));


Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: ['one-photos-photo-list-model', 'one-photos-test-shared', 'test']
});
