/*
 * A scrollable datatable based on some of the functionality of
 * slickgrids but using tables.
 *
 */

Y.namespace('u1');
Y.u1.TableScroll = Y.Base.create("TableScroll", Y.Widget, [], {
    initializer: function() {
        // Set width of scrollbars
        this.set("nScrollbarWidth", this.getScrollbarWidth());

        // Init vars
        var oNodeContainer = this.get("oNodeContainer"),
            oRowSpec = this.get("oRowSpec");

        // Set attrs for table pieces
        this.set("oNodeTable", oNodeContainer.one("table"));
        this.set("oNodeTbody", oNodeContainer.one("tbody"));
        this.set("oNodeThead", oNodeContainer.one("thead"));
        this.set("oNodeTfoot", oNodeContainer.one("tfoot"));

        // Init a stylenode for us to manipulate to provide column updates.
        this.styleNode = new Y.u1.StyleNode();

        // Row Cache
        this.rows = {};
        // dataFetchFlag
        this.timer = null;
        // Data Cache
        this.dataCache = new Y.Cache({max: this.get("nMaxDataCacheItems"), uniqueKeys: true});

        if (!oRowSpec || !oRowSpec.preDataTemplate || !oRowSpec.postDataTemplate) {
            window.alert("Please define oRowSpec attribute as an object with 'preDataTemplate' and 'postDataTemplate' template node properties.");
            return;
        } else {
            this.set('sRowPreDataTemplate', Y.one(oRowSpec.preDataTemplate).getContent());
            this.set('sRowPostDataTemplate', Y.one(oRowSpec.postDataTemplate).getContent());
        }
    },
    // Debug Mode
    debugInit: function() {
        var oNodeDebugCont = Y.Node.create("<div class='isroll-debug'><dl></dl></div>"),
            oNodeDl = oNodeDebugCont.one("dl"),
            oNodeNumTrs = Y.Node.create('<dt>Number Trs: </dt><dd id="tablescroll-ntrs"></dd>'),
            oNodeNumTotalRecords = Y.Node.create('<dt>Total No. of Records: </dt><dd id="tablescroll-totrs"></dd>'),
            oNodeNumDataPages = Y.Node.create('<dt>Number Data Pages: </dt><dd id="tablescroll-ndps"></dd>'),
            oNodeCurDataPages = Y.Node.create('<dt>Current Data Page: </dt><dd id="tablescroll-cdp"></dd>'),
            oNodeLastScrollTop = Y.Node.create('<dt>LastScrollTop: </dt><dd id="tablescroll-lst"></dd>'),
            oNodeTotalScrollHeight = Y.Node.create('<dt>Total Scroll Height: </dt><dd id="tablescroll-th"></dd>'),
            oNodeRangeVisibleTop = Y.Node.create('<dt>Range visible Top: </dt><dd id="tablescroll-orv-top"></dd>'),
            oNodeRangeVisibleMiddle = Y.Node.create('<dt>Range visible Middle: </dt><dd id="tablescroll-orv-middle"></dd>'),
            oNodeRangeVisibleBottom = Y.Node.create('<dt>Range visible Bottom: </dt><dd id="tablescroll-orv-bottom"></dd>'),
            toAppend = [oNodeNumTrs, oNodeNumDataPages, oNodeCurDataPages,
                        oNodeLastScrollTop, oNodeNumTotalRecords, oNodeTotalScrollHeight,
                        oNodeRangeVisibleTop, oNodeRangeVisibleMiddle, oNodeRangeVisibleBottom],
            i;

        for (i in toAppend) {
            oNodeDl.append(toAppend[i]);
        }

        Y.Node.one("body").append(oNodeDebugCont);
        oNodeDebugCont.setStyles({
            "position": "fixed",
            "z-index": 10,
            "top": "5px",
            "left": "5px",
            "width": "300px",
            "height": "250px",
            "background": "#ccc"
        });
        this.after("nLastScrollTopChange", function(e) {
            Y.Node.one("#tablescroll-lst").setContent(e.newVal);
        }, this);
        this.after("aCurrentDataPageChange", function(e) {
            Y.Node.one("#tablescroll-cdp").setContent(e.newVal);
        }, this);
        this.after("nNumTrsChange", function(e) {
            Y.Node.one("#tablescroll-ntrs").setContent(e.newVal);
        }, this);
        this.after("nTotalDataPagesChange", function(e) {
            Y.Node.one("#tablescroll-ndps").setContent(e.newVal);
        }, this);
        this.after("nScrollbarHeightChange", function(e) {
            Y.Node.one("#tablescroll-th").setContent(e.newVal);
        }, this);
        this.after("nTotalRecordsChange", function(e) {
            Y.Node.one("#tablescroll-totrs").setContent(e.newVal);
        }, this);
        this.after("nRangeVisibleTopChange", function(e) {
            Y.Node.one("#tablescroll-orv-top").setContent(e.newVal);
        }, this);
        this.after("nRangeVisibleMiddleChange", function(e) {
            Y.Node.one("#tablescroll-orv-middle").setContent(e.newVal);
        }, this);
        this.after("nRangeVisibleBottomChange", function(e) {
            Y.Node.one("#tablescroll-orv-bottom").setContent(e.newVal);
        }, this);
        this.set("nNumTrs", this.get("oNodeContainer").all("tbody tr").size());
    },
    renderUI: function() {

        var bDebugMode = this.get("bDebugMode"),
            nTheadHeight,
            oNodeContainer = this.get("oNodeContainer"),
            oNodeTheadClone,
            oNodeTheadCloneDiv,
            oNodeTheadCloneTable,
            oNodeThead,
            oNodeTable;

        if (bDebugMode === true) {
            this.debugInit();
        }

        oNodeThead = this.get("oNodeThead");
        oNodeTable = this.get("oNodeTable");

        // Get an identifier on the table so we can directly attach styles.
        if (!oNodeContainer.get("id")) {
            oNodeContainer.set("id", Y.guid());
        }

        // Thead cloning
        oNodeTheadClone = oNodeThead.cloneNode(true);
        this.set("oNodeTheadClone", oNodeTheadClone);

        // Clone our "real" thead and place it above the main table of content
        // Yeah this feels wrong, but there's little other options and building this
        // entire thing out of lists feels even more of a violation.
        nTheadHeight = oNodeThead.get("offsetHeight");
        oNodeTheadCloneDiv = Y.Node.create("<div class='tablescroll-thead'><table></table></div>");
        if (Y.UA.ie === 7) {
            oNodeTheadCloneDiv.setStyle("zoom", 1);
        }
        oNodeTheadCloneTable = oNodeTheadCloneDiv.one("table");
        oNodeTheadCloneTable.append(oNodeTheadClone);
        oNodeContainer.insertBefore(oNodeTheadCloneDiv, oNodeContainer);
        oNodeThead.remove();

        // Set styles on the Container Node
        oNodeContainer.setStyles({
            "height": this.get('nContainerHeight'),
            "position": "relative",
            "overflowX": "hidden",
            "overflowY": "scroll"
        });

        // Set some initial styles on our table
        oNodeTable.setStyles({"height": -nTheadHeight,
                              "display": "block"});
    },
    bindUI: function() {
        var oNodeContainer = this.get("oNodeContainer"),

        // Scroll and mousewheel events - these are checked for actual
        // scrolling in the callback
        scrollFunc;

        if (Y.UA.mobile > 0) {
            scrollFunc = Y.throttle(Y.bind(this.onScroll, this), 100);
        } else {
            scrollFunc = Y.bind(this.onScroll, this);
        }
        Y.on("mousewheel", scrollFunc);
        oNodeContainer.on("scroll", scrollFunc);

        // Update the table height when we know what data we are dealing with.
        // The first time this happens will be after the first data set loads.
        this.after("nTotalRecordsChange", this.onTotalRecordsChange, this);
        // Watch for the page change so we can work out what data we need to get
        this.after("aCurrentDataPageChange", this.onDataPageChange, this);

        this.after('visibleChange', this.afterVisibleChange, this);

        Y.on("windowresize", Y.bind(this.syncUI, this));

    },
    syncUI: function() {
        if (this.get("visible") === true) {
            this.renderWidget();
            this.setColumnWidths();
        }
    },
    setColumnWidths: function() {
        var aColumnSpec = this.get("aColumnSpec"),
            aTdRules,
            aEllipsisRules,
            aStyleRules,
            nHorizPaddingAndBorder,
            nFlexWidth,
            nFlexWidthRemainder,
            nFixedWidthTotal = 0,
            nLastWidth = 0,
            nPercWidthTotal = 0,
            nPixelWidth,
            nVertPaddingAndBorder,
            nWidth = null,
            oNodeTable = this.get("oNodeTable"),
            nCurrentTableWidth = oNodeTable.get("offsetWidth"),
            oNodeListTds,
            oNodeCurTd,
            oNodeFirstTd,
            oNodeStyle,
            sContainerId = this.get("oNodeContainer").get("id"),
            sExtraRules,
            sOrigContent,
            i,
            j;

        for (i=0, j=aColumnSpec.length; i < j; i++) {
            nPixelWidth = aColumnSpec[i].width;
            nFlexWidth = aColumnSpec[i].flexwidth;

            if (typeof nPixelWidth !== "undefined") {
                nFixedWidthTotal += parseInt(nPixelWidth, 10);
            }
            if (typeof nFlexWidth !== "undefined") {
                nPercWidthTotal += parseInt(nFlexWidth, 10);
            }
        }

        /**
          if (nPercWidthTotal > 100) {
          Y.log("Iscroll: flexwidths defined in aColSpec cannot total more than 100% - proceeding with unknown results.");
          }
         **/

        // Get the remaining width we have to play with for the flexible columns.
        nFlexWidthRemainder = nCurrentTableWidth - nFixedWidthTotal;

        // Get NodeList containing first row of Tds.
        oNodeListTds = this.get("oNodeTbody").all("tr:first-child td");
        if (oNodeListTds.size() === 0) {
            return;
        }

        // Webkit seems to have trouble calculating the padding + border unless something is taking up vertical space
        // To get around that we drop a div set to the height of the row and then swap it back out.
        oNodeFirstTd = oNodeListTds.item(0);
        sOrigContent = oNodeFirstTd.getContent();
        oNodeFirstTd.setContent(Y.Node.create("<div style='height:" + this.get("nRowHeight") + "px'></div>"));
        nVertPaddingAndBorder = oNodeFirstTd.get("offsetHeight") - parseInt(oNodeFirstTd.getComputedStyle("height"), 10);
        oNodeFirstTd.setContent(sOrigContent);

        // Initial Styles - this truncates the table-cells + ensure the height matches that of the rows.
        aTdRules = [
            "white-space: nowrap;",
            "position: absolute;",
            "overflow: hidden;",
            "height: "+ (this.get("nRowHeight") - nVertPaddingAndBorder)+ "px;"
        ];

        aStyleRules = [
            "#" + sContainerId + " table" +  "{ table-layout:fixed }",
            "#" + sContainerId + " tr" +  "{ width: " + nCurrentTableWidth +"px; }",
            "#" + sContainerId + " td { "+ aTdRules.join(" ") +"}"
        ];

        // Some rules for displaying an ellipsis for truncated text.
        // Only used if ellipsis = true in aColumnSpec
        aEllipsisRules = [
            "text-overflow: ellipsis;",
            "-o-text-overflow: ellipsis;",
            "-ms-text-overflow: ellipsis;"
        ];

        for (i=0, j=aColumnSpec.length; i < j; i++) {
            nPixelWidth = aColumnSpec[i].width;
            oNodeCurTd = oNodeListTds.item(i);

            nFlexWidth = aColumnSpec[i].flexwidth;
            if (typeof nPixelWidth !== "undefined") {
                nWidth = nPixelWidth;
            }
            if (typeof nFlexWidth !== "undefined") {
                nWidth = (nFlexWidthRemainder / 100) * nFlexWidth;
            }
            // Allow for padding + border in the width
            nHorizPaddingAndBorder = (oNodeCurTd.get("offsetWidth") - parseInt(oNodeCurTd.getComputedStyle("width"), 10));
            nWidth = nWidth - nHorizPaddingAndBorder;
            sExtraRules = "";
            if (aColumnSpec[i].ellipsis === true) {
                sExtraRules = aEllipsisRules.join(" ");
            }
            aStyleRules.push("#" + sContainerId + " .c" + (i+1) + "{ left: " + nLastWidth +"px; width:" + nWidth +"px; "+sExtraRules+"}");

            // Allow for padding + border for the next left value
            nLastWidth += nWidth + nHorizPaddingAndBorder;
        }

        // Blow away an existing styleNode so we can recreate it.
        oNodeStyle = this.styleNode.get("styleNode");
        if (oNodeStyle !== null) {
            oNodeStyle.remove();
        }
        this.styleNode.create(aStyleRules);
        this.updateThWidths();
    },
    updateThWidths: function() {
        var tableCells = this.get("oNodeTbody").all("tr:first-child td"),
            tableHeadings = this.get("oNodeTheadClone").all("th"),
            elmWidth,
            numTds = tableCells.size();
        tableCells.each(function(elm, i){
            // Bail on the last th as we want this to fill all space available.
            if (i+1 === numTds) {
                return;
            }
            elmWidth = elm.get("offsetWidth");
            if (elmWidth > 0) {
                tableHeadings.item(i).set("offsetWidth", elmWidth);
            }
        }, this);
    },
    onTotalRecordsChange: function(e){
        if (e.newVal === e.prevVal) {
            return;
        }
        var nScrollbarHeight = e.newVal * this.get('nRowHeight');
        this.get("oNodeTable").setStyle("height", nScrollbarHeight);
        this.set("nScrollbarHeight", nScrollbarHeight);
        this.set("nTotalDataPages", Math.ceil(nScrollbarHeight / (this.get("nRowHeight") * this.get("nRecordLimit"))));
        if (typeof e.prevVal !== "undefined") {
            this.dataCache.flush();
            this.syncUI();
        }
    },
    onDataPageChange: function(e){

        // If prev value matches the new value then bail
        if (e.prevVal === e.newVal) {
            return;
        }

        var aCurrentDataPage = e.newVal;

        // If timer is already in progress abort it.
        if (this.timer !== null) {
            clearTimeout(this.timer);
        }

        // Set-up a timer to check we're still on the same page after a delay.
        // This helps to avoid fetching data unecessarily if we are scrolling fast.
        this.timer = setTimeout(Y.bind(function(o){
            var pageList = [],
                nRecordLimit = this.get("nRecordLimit"),
                aCurrentDataPage = o.currentPage,
                offset, limit, page,
                i,
                j;

            if (this.get("aCurrentDataPage") === aCurrentDataPage) {

                // Build a list of what pages we require - checking that the cache value is undefined
                // It will be null if fetching is already in progress in which case we won't fetch it.
                for (i=0, j=aCurrentDataPage.length; i < j; i++) {
                    page = aCurrentDataPage[i];
                    // yui3's cache returns null if the key isn't present
                    if (this.dataCache.retrieve("is-" + page) === null) {
                        pageList.push(page);
                    }
                }

                // If there's nothing to fetch then exit at this point.
                if (pageList.length === 0){
                    return;
                }

                // The first item in the list will always be the earliest page we need.
                offset = (pageList[0] - 1) * nRecordLimit;
                limit = (nRecordLimit * pageList.length);

                // Mark this as pending data - so we don't make subsequent reqs for it inbetween now
                // and actually receiving the data.
                for (i=0, j=pageList.length; i<j; i++) {
                    this.dataCache.add("is-" + pageList[i], "pending");
                }
                // Merge the updated data.
                this.fetchData(offset, limit);
            }
            this.timer = null;
        }, this, {
            "currentPage": aCurrentDataPage
        }), this.get("nDataFetchDelay"));

    },
    /* Called following a successful first data load */
    onInitialData: function(){
        // Run onScroll - this will deal with the issue of loading a
        // non-zero scroll start point.
        this.onScroll();
    },
    /* Abstracted method to get data - assumes the datasource method supplied
     * in the config accepts an object with success/failure/offset/limit keys
     *
     * offset is the record to start from
     * limit is the limit of the number of records to fetch.
     * */
    fetchData: function(offset, limit) {
        this.get("oDataSource")[this.get("sDataSourceMethod")]({
            "success": Y.rbind(this.onIOSuccess, this, offset, limit),
            "failure": Y.rbind(this.onIOFailure, this, offset, limit),
            "offset": offset,
            "limit": limit
        });
    },
    /* following a successful data load */
    onIOSuccess: function(e, offset, limit) {
        var nRecordLimit = this.get("nRecordLimit"),
            records = this.get("fRecordListSchema")(e),
            startPage = Math.ceil((offset / nRecordLimit) + 1),
            nTotalRecords;

        // We always check for changes to the total number of records.
        // So we can update for new data as it's added behind the scenes.
        nTotalRecords = this.get("fRecordTotalSchema")(e);
        this.set("nTotalRecords",  nTotalRecords);
        this.set("nTotalPages", nTotalRecords / this.get("nRecordLimit"));

        if (!this.get("bLoadedInitialData")){
            this.onInitialData(e);
        }

        // Stash the current dataset away under the page num as a key.
        // Handle the case where we fetch two pages at once for when
        // we are straddling data pages.
        if (limit > nRecordLimit) {
            this.dataCache.add("is-" + startPage, records.slice(0, nRecordLimit));
            this.dataCache.add("is-" + (startPage+1), records.slice(nRecordLimit, limit));
        } else {
            this.dataCache.add("is-" + startPage, records);
        }

        this.clearRows();
        this.renderWidget();

        if (!this.get("bLoadedInitialData")){
            this.setColumnWidths();
            this.set("bLoadedInitialData", true);
        }

    },
    /* Failure to get data
     * This could simply mean network is temporarily bad
     * TODO: This function should allow for a retry
     * TODO: This needs to clear null values in the cache so retries can happen.
     * */
    onIOFailure: function() {
        Y.log("oh dear something went wrong here");
    },
    /* Reset the Data cache */
    clearDataCache: function(){
        this.dataCache.flush();
    },
    // Clear out the rows
    clearRows: function() {
        for (var i in this.rows){
            this.removeRow(i);
        }
    },
    removeRow: function(index) {
        // Remove the node
        this.rows[index].remove(true);
        // Remove from the cache
        delete this.rows[index];
    },
    /* Based on the current data page get the data for this row num */
    getRowData: function(nRow) {
        var nRecordLimit = this.get("nRecordLimit"),
            page = Math.ceil((nRow + 1) / nRecordLimit),
            offsetIndex = nRow,
            dataset;

        page = page > 0 ? page : 1;
        if (page > 1) {
            offsetIndex = Math.floor((nRow - ((page - 1) * nRecordLimit)));
        }

        // Y.log("rownum: " + (nRow + 1) + " nRow: " + nRow + " page: " + page + " offsetIndex: " + offsetIndex  );

        dataset = this.dataCache.retrieve("is-" + page);
        if (dataset && dataset.response !== "pending" && dataset.response[offsetIndex]) {
            return dataset.response[offsetIndex];
        }
    },
    /* Renders the current view of the visible data */
    renderWidget: function() {
        var oNodeContainer = this.get("oNodeContainer"),
            nRowHeight = this.get("nRowHeight"),
            nScrollTop = oNodeContainer.get("scrollTop"),
            nContainerHeight = this.get("nContainerHeight"),
            nBufferHeight = nContainerHeight,
            oNodeTbody = this.get("oNodeTbody"),
            bDebugMode = this.get("bDebugMode"),
            nTotalRecords = this.get('nTotalRecords'),
            top = Math.floor((nScrollTop-nBufferHeight)/nRowHeight),
            bottom = Math.ceil((nScrollTop + nContainerHeight + nBufferHeight)/nRowHeight),
            middle,
            node,
            i;


        this.top = top = Math.max(0, top);
        this.bottom = bottom = Math.min(this.get("nScrollbarHeight")/nRowHeight, bottom);
        middle = Math.ceil(top + ((bottom - top)/2));

        // Work out what data page(s) we are currently accessing
        // This will work out the data page that the top of the rendered rows
        // falls in as well as the ones that the bottom of the rendered rows sees.
        // This way if we load and we're straddling two consecutive pages of data we
        // make one request for data spanning those two pages.
        this.set("aCurrentDataPage", this.getCurrentDataPage());

        if (bDebugMode) {
            this.set("nRangeVisibleTop", top);
            this.set("nRangeVisibleMiddle", middle);
            this.set("nRangeVisibleBottom", bottom);
        }

        // Trash Cached rows above and below
        // where we are.
        for (i in this.rows){
            if (i < top || i > bottom) {
                this.fire("preRowDestroyed", {"node": this.rows[i]});
                this.rows[i].remove(true);
                delete this.rows[i];
            }
        }

        // render + append + cache the rows between top and bottom values
        for (i=top; i<=bottom; i++) {
            if (i > (nTotalRecords - 1)) {
                continue;
            }
            if (!this.rows[i]) {
                node = this.rows[i] = this.renderRow(i);
            } else {
                node = this.rows[i];
            }
            oNodeTbody.append(node);
        }

        if (bDebugMode) {
            // Get the number of trs currently in play.
            this.set("nNumTrs", this.get("oNodeContainer").all("tbody tr").size());
        }
    },
    /* Renders an individual row */
    renderRow: function(nRow) {
        var currentRow = { "tablescroll-rownum": nRow + 1 },
            nRowHeight = this.get("nRowHeight"),
            node, oRowData, rowData, fCustomRowRenderer;

        oRowData = this.getRowData(nRow);
        if (oRowData){
            rowData = Y.merge(oRowData, currentRow);
            fCustomRowRenderer = this.get("fCustomRowRenderer");
            if (fCustomRowRenderer && typeof fCustomRowRenderer === "function") {
                node = fCustomRowRenderer(rowData, nRow, this);
            } else {
                node = Y.Node.create(Y.substitute(this.get("sRowPostDataTemplate"), rowData, function(k,v){return Y.Escape.html(v);}));
            }
            this.fire("rowDataRendered", { "node": node, "data": oRowData });
        } else {
            node = Y.Node.create(Y.substitute(this.get("sRowPreDataTemplate"), currentRow, function(k,v){return Y.Escape.html(v);}));
            this.fire("rowRendered", { "node": node });
        }
        node.setStyles({
            "top": nRow * nRowHeight + "px",
            "height": nRowHeight  + "px",
            "position": "absolute"
        });
        return node;
    },
    getCurrentDataPage: function(){
        var nRowHeight = this.get("nRowHeight"),
            nRecordLimit = this.get("nRecordLimit"),
            pxHeightDataPage = (nRecordLimit * nRowHeight),
            topPage, bottomPage, dataPages;

        topPage = Math.ceil((this.top * nRowHeight) / pxHeightDataPage);
        topPage = topPage > 0 ? topPage : 1;
        bottomPage = Math.ceil((this.bottom * nRowHeight) / pxHeightDataPage);

        topPage = topPage > 0 ? topPage : 1;
        bottomPage = bottomPage > 0 ? bottomPage : 1;

        // Only pass a list of one page if they're the same.
        if (topPage === bottomPage) {
            dataPages = [topPage];
        } else {
            dataPages =  [topPage, bottomPage ];
        }
        return dataPages;
    },
    onScroll: function(){
        if (this.get("visible") === true) {
            // Handle scroll events but ignoring those fired by viewport scrollig.
            // To work that out we check for changes in scrollTop on the container.
            var scrollTop = this.get("oNodeContainer").get("scrollTop");

            // When initialised nLastScrollTop is null so this will execute
            // on first load too.
            if (this.get("nLastScrollTop") !== scrollTop) {
                // Work out what page of data we are on.
                this.renderWidget();
            }
            this.set("nLastScrollTop", scrollTop);
        }
    },
    /* A way to work out a scrollbar width for most browser implementations
     * TODO: This is likely to fail for UAs that don't provide scrollbars until
     * interactions take place e.g. IOS
     * */
    getScrollbarWidth: function() {
        var scrollbarWidth = 0,
            div = Y.Node.create('<div style="width:50px;height:50px;position:absolute;top:-999em;left:999em;"><div style="height:100px; width: 100%"></div></div>'),
            w1, w2;
        Y.one('body').append(div);
        w1 = parseInt(div.one("div").get("offsetWidth"), 10);
        div.setStyle('overflow-y', 'scroll');
        w2 = parseInt(div.one("div").get("offsetWidth"), 10);
        div.remove();
        scrollbarWidth = w1 - w2;
        // Return something for a zero result - this means that for OSX where scrollbars are
        // invisible until interacted with we will still have some space for it.
        return (scrollbarWidth !== 0) ? scrollbarWidth : 18;
    }
}, {
    NAME: "tablescroll",
    ATTRS: {
        /* The column definition */
        aColumnSpec: { value: null },
        oRowSpec: { value: null },
        bDebugMode: { value: false },
        bLoadedInitialData: { value: false },
        /* function that can data the data returned by fGetData and
         * return the list of records to utilise */
        fRecordListSchema: { value: null },
        /* function that can take the data returned by dGetData and
         * return the total number of records */
        fRecordTotalSchema: { value: null },
        /* Custom row renderer */
        fCustomRowRenderer: { value: null },
        /* Height of container in pixels */
        nContainerHeight: { value: 400 },
        /* The current page of data we are on */
        aCurrentDataPage: { value: [1,1] },
        /* A delay to introduce before we fetch data so we
         * don't fetch things when we're scrolling fast */
        nDataFetchDelay: { value: 250 },
        /* The current scrollTop value */
        nLastScrollTop: { value: null },
        /* The limit of records to fetch */
        nRecordLimit: { value: 50 },
        /* The height of the table rows in px */
        nRowHeight: { value: 50 },
        /* The scrollbar width is set here */
        nScrollbarWidth: { value: null },
        /* The scrollbar height is set here - this is based on num records + row height */
        nScrollbarHeight: { value: null },
        /* The number of tr objects used */
        nNumTrs: { value: null },
        /* Number of datasets to keep in the cache */
        nMaxDataCacheItems: { value: 10 },
        /* The total num of data pages there are */
        nTotalDataPages: { value: null },
        nRangeVisibleTop: { value: null },
        nRangeVisibleMiddle: { value: null },
        nRangeVisibleBottom: { value: null },
        /* Node object of the widget container */
        oNodeContainer: {
            setter: function(sel) {
                var n = Y.one(sel);
                if (!n) {
                    throw new Error('TableScroll.oNodeContainer - invalid selector provided: ' + sel);
                }
                return n;
            }
        },
        /* Table parts as Nodes */
        oNodeTable: { writeOnce: true },
        oNodeTbody: { writeOnce: true },
        oNodeTfoot: { writeOnce: true },
        oNodeThead: { writeOnce: true },
        oNodeTheadClone: { writeOnce: true },

        /* A DataIO instance for getting data for the table */
        oDataSource: { value: null },
        /* The method of the dataIO instance to use for fetching data */
        sDataSourceMethod: { value: null },
        /* The row template content as a string */
        sRowPreDataTemplate: { value: null },
        sRowPostDataTemplate: { value: null }
    }
});
