
var UploaderFollow,

    EVENT = 'fileuploadstart';

/**
* follows one-uploader file upload progression by scrolling
*
* @module one-uploader-follow
*/
UploaderFollow = Y.Base.create('oneUploaderFollow', Y.Base, [], {


    /**
    * keeps track of the previous scroll value in px
    *
    * @property _previousScrollValue
    */
    _previousScrollValue: null,

    /**
    * prop for animation
    *
    * @property _scrollAnim
    */
    _scrollAnim: new Y.Anim({
        duration: 0.1
    }),

    initializer: function () {

        this.after('render', this.bind);

    },

    bind: function () {

        this.on('followChange', this._followChange);

        if(this.get('follow') && this.get('hostNode')) {

            this.on(EVENT, this._followFile, this);
            this.get('hostNode').on('scroll', this._checkScroll, this);

            if(this.get('animated')) {
                this._scrollAnim.set('node', this.get('hostNode'));
                this._scrollAnim.after('toChange', function () {
                    this.run();
                });
            }

        }

    },

    startFollowing: function () {
        this.set('follow', true);
    },

    stopFollowing: function () {
        this.set('follow', false);
    },

    /**
    * bind/unbind relevant events
    *
    * @method _followChange
    * @protected
    */
    _followChange: function (e) {

        if(e.newVal) {
            this._bind();
        } else {
            this._unBind();
        }

    },

    /**
    * If user scrolled, unbind all follow handlers
    *
    * @method _checkScroll
    * @protected
    */
    _checkScroll: function (event) {

        var targetScrollTop = event.target.get('scrollTop');

        if(!this._previousScrollValue) {
            this._previousScrollValue =  targetScrollTop;
        }

        // no upwards scroll (against the flow), turn following off
        if(targetScrollTop < (this._previousScrollValue)) {
            this.stopFollowing();
        }

        this._previousScrollValue = targetScrollTop;

    },

    /**
    * unbind events
    *
    * @method _unBind
    * @protected
    */
    _unBind: function () {

        this.detach(EVENT, this._followFile);
        this.get('hostNode').detach('scroll', this._checkScroll);
        this._scrollAnim.detach();
    },

    /**
    * Follows file progression
    *
    * @method _followFile
    * @protected
    */
    _followFile: function (event) {

        var row         = event.file._node,
            rowHeight   = parseInt(row.getComputedStyle('height'), 10),
            rowOffset   = row.get('offsetTop'),
            host        = this.get('hostNode'),
            hostHeight  = parseInt(host.getComputedStyle('height'), 10),
            target;

        // do the actual scrolling
        if(rowOffset + rowHeight > hostHeight) {
            target = rowOffset + rowHeight - hostHeight;

            if(this.get('animated')) {

                // if there is a running animation, stop it and forward to last frame
                if(this._scrollAnim.get('running')) {
                    this._scrollAnim.stop(true);
                }

                this._scrollAnim.set('to', { scrollTop: target });

            } else {
                host.set('scrollTop', target);
            }


        }


    }

}, {

    ATTRS: {
        animated: { value: true },
        follow: { value: true },
        hostNode: {
            value: '.yui3-followscrolling',
            setter: function (v) {
                return Y.Lang.isString(v) ? Y.one(v) : v;
            }
        }
    }

});

Y.namespace('U1.Widgets').UploaderFollow = UploaderFollow;
