YUI.add('one-uploader-test', function (Y) {

var suite;


suite = new Y.Test.Suite('one-uploader');

suite.add(new Y.Test.Case({

    name: 'Base',

    setUp: function () {

        this.app = new Y.U1.Widgets.Uploader();

        this.filearray = [];
        for (var i = 0; i <= 2; i++) {
            this.filearray.push(new Y.FileHTML5({name: i + ".jpg", size: i*1000, type: "jpg"}));
        }

        this.app.render();

        this.queue = new Y.UploaderHTML5.Queue({
            simUploads: 1,
            //errorAction: Y.Uploader.Queue.RESTART_ASAP,
            fileList: this.filearray,
            uploadURL: "test.html"
        });

       // set filelist
        this.app.set('fileList', this.filearray);
        this.app.fire('fileselect', {
            fileList: this.filearray
        });

    },

    tearDown: function () {
        if(this.app) {
            this.app.destroy();
        }
        delete this.app;
        delete this.filearray;
        delete this.queue;
    },

    "instance ok": function () {
        Y.Assert.isInstanceOf(Y.U1.Widgets.Uploader, new Y.U1.Widgets.Uploader());
    },

    "test getFile": function () {
        Y.Assert.areSame(this.filearray[0], this.app.getFile(this.filearray[0].get('id')));
    },

    "test getUploadsInProgress": function () {
        this.app.set('simLimit', 3);
        this.app.uploadAll();
        Y.Assert.areSame(this.app.get('fileList').length, this.app.getUploadsInProgress().length);
    },

    "test getHaveUploadsInProgress": function () {
        this.app.uploadAll();
        Y.Assert.isTrue(this.app.getHaveUploadsInProgress());
    },

    "test getHaveUploadsInProgress for only one file": function () {
        this.app.set('fileList', [this.filearray[0]]);
        this.app.uploadAll();
        Y.Assert.isTrue(this.app.getHaveUploadsInProgress());
    },

    "test getPendingFiles": function () {
        Y.Assert.areSame(this.app.get('fileList').length, this.app.getPendingFiles().length);
        this.app.set('fileList', []);
        Y.Assert.areSame(0, this.app.getPendingFiles().length);
    },

    "test getHavePendingFiles": function () {
        Y.Assert.isTrue(this.app.getHavePendingFiles());
        this.app.set('fileList', []);
        Y.Assert.isFalse(this.app.getHavePendingFiles());
    },

    "test getHaveNoFiles": function () {
        Y.Assert.isFalse(this.app.getHaveNoFiles());
        this.app.set('fileList', []);
        this.queue.destroy();
        Y.Assert.isTrue(this.app.getHaveNoFiles());
    },

    "test getFilesTotals": function () {
        var file1 = new Y.FileHTML5({name: 'foo.jpg', size: 21, type: 'jpg'}),
            file2 = new Y.FileHTML5({name: 'bar.jpg', size: 21, type: 'jpg'});

        this.app.set('fileList', [file1, file2]);

        Y.Assert.areSame(42, this.app.getFilesTotals().size);
        Y.Assert.areSame(2, this.app.getFilesTotals().count);
    },

    "test getFriendlySize": function () {
        Y.Assert.areSame('1000 B', this.app.getFriendlySize(1000));
    },

    "test cancelFile": function () {

        // make queue active and push to queuedFiles
        this.app.queue = this.queue;
        this.app.queue._currentState = Y.UploaderHTML5.Queue.UPLOADING;
        this.app.uploadAll();

        this.app.cancelFile(this.filearray[1]);

        Y.ArrayAssert.doesNotContain(this.filearray[1], this.app.queue.queuedFiles);
        Y.ArrayAssert.contains(this.filearray[2], this.app.queue.queuedFiles);
    },

    "test getFilesNotUploaded": function () {

        var file1 = new Y.FileHTML5({name: 'foo.jpg', size: 21, type: 'jpg'}),
            file2 = new Y.FileHTML5({name: 'bar.jpg', size: 21, type: 'jpg'});

        this.app.set('fileList', [file1, file2]);

        Y.Assert.areSame(2, this.app.getFilesNotUploaded().length, 'no files uploaded');

        this.app.fire('uploadcomplete', {
            file: file1
        });

        Y.Assert.areSame(1, this.app.getFilesNotUploaded().length, 'one file left');

    },

    "test getFilesNotUploaded after cancel": function () {

        var fileList = this.app.get('fileList');

        Y.Assert.areSame(3, this.app.getFilesNotUploaded().length);

        // finish two files, cancel one
        this.app.fire('uploadcomplete', {
            'file': fileList[0]
        });
        this.app.fire('uploadcomplete', {
            'file': fileList[1]
        });

        this.app.cancelFile(fileList[2]);

        Y.Assert.areSame(0, this.app.getFilesNotUploaded().length);

    },

    "all files cancelled does not create stale upload window, fires incomplete": function () {
        var eventCalled = false;

        // make queue active and push to queuedFiles
        this.app.queue = this.queue;
        this.app.queue._currentState = Y.UploaderHTML5.Queue.UPLOADING;
        this.app.uploadAll();

        // this should be called, so let us just detach the defaul one
        // and create our own
        this.app.detach('incomplete');
        this.app.on('incomplete', function () {
            eventCalled = true;
        });

        // cancel all uploads while they are uploading
        Y.Array.each(this.filearray, function (file) {
            this.app.cancelFile(file);
        }, this);


        Y.Assert.isTrue(eventCalled, 'incomplete event called');

    },

    "pending uploads reported to user": function () {
        var status          = this.app.get('status'),

            undefinedRe     = new RegExp('undefined'),
            filesRe         = new RegExp(this.app.get('fileList').length + '.+ files');

        Y.Assert.isFalse(undefinedRe.test(status)    , 'undefined in status');
        Y.Assert.isTrue(filesRe.test(status)    , 'did not find amount of files');
    },

    "no NaNs in status": function () {
        var zeroLengthFile      = new Y.FileHTML5({ name: 'zero.jpg', size: 0, type: 'jpg' }),
            nullLengthFile      = new Y.FileHTML5({ name: 'null.jpg', size: null, type: 'jpg' }),
            undefinedLengthFile = new Y.FileHTML5({ name: 'undefined.jpg', size: undefined, type: 'jpg' }),
            negativeLengthFile  = new Y.FileHTML5({ name: 'negative.jpg', size: -1, type: 'jpg' }),

            NaNre               = new RegExp('NaN');

        Y.each([zeroLengthFile, nullLengthFile, undefinedLengthFile, negativeLengthFile], function (file) {
            this.app.set('fileList', [file]);
            this.app.fire('fileselect');
            Y.Assert.isFalse(NaNre.test(this.app.get('status')));
        }, this);
    },

    "lacking a proper percentage, show something else": function () {
        var badRe               = new RegExp('NaN|null|undefined'),
            values              = [NaN, null, undefined, "NaN", "null", "undefined"];

        Y.each(values, function (val) {
            this.app.fire('totaluploadprogress', {
                percentLoaded: val
            });
            Y.Assert.isFalse(badRe.test(this.app.get('status')), 'no bad values in _totalProgressNode');
        }, this);

    },

    "large singular files should report percentage": function () {

        var file = new Y.FileHTML5({name: 'foo.jpg', size: 100, type: 'jpg'}),
            loaded = 0;

        this.app.set('fileList', [file]);
        this.app.fire('fileselect');

        this.app.on('totaluploadprogress', function (e) {
            loaded = e.percentLoaded;
        });

        this.app.uploadAll();

        file.fire('uploadprogress', {
            bytesLoaded: 5
        });

        // 5 bytes of 100 bytes is 5 percent
        Y.Assert.areSame(5, loaded);
    },

    "changing drag drop area fully detaches and reattaches our events": function () {

        var dd = Y.Node.create('<div class="foo" />');

        // dd has a drop evt listener
        this.app.set('dragAndDropArea', dd);
        Y.Assert.areSame(1, this.app.getEvent('drop')._subscribers.length);

        // dd has no drop listener
        this.app.set('dragAndDropArea', null);
        Y.Assert.areSame(0, this.app.getEvent('drop')._subscribers.length);

        // listener is reattached
        this.app.set('dragAndDropArea', dd);
        Y.Assert.areSame(1, this.app.getEvent('drop')._subscribers.length);

    },

    "test beforeunload": function () {

        Y.Assert.isNotNull(window.onbeforeunload);

        Y.Assert.areSame('Discard 3 pending files?', this.app._beforeUnload({}));

        this.app.destroy();

        Y.Assert.isNull(window.onbeforeunload);
    }


}));

suite.add(new Y.Test.Case({

    name: 'File',

    setUp: function () {

        this.app = new Y.U1.Widgets.Uploader();
        this.app._bindUploaderEvents();
        this.app.set('fileList', [
            new Y.FileHTML5({name:'foo'})
        ]);
        this.file = this.app.get('fileList')[0];

    },

    tearDown: function () {
        this.app.destroy();
    },


    "state: progress": function () {

        this.app.fire('uploadprogress', {
            file: this.file
        });

        Y.Assert.areSame('UPLOADING', this.file.get('status'));

    },

    "state: complete": function () {

        this.app.fire('uploadcomplete', {
            file: this.file
        });

        Y.Assert.areSame('COMPLETE', this.file.get('status'));

    },

    "state: failed": function () {

        this.app.fire('uploaderror', {
            file: this.file
        });

        Y.Assert.areSame('FAILED', this.file.get('status'));

    },

    "state: cancelled": function () {

        this.app.fire('filecancel', {
            file: this.file
        });

        Y.Assert.areSame('CANCELLED', this.file.get('status'));

    },

    "state: started": function () {

        this.app.fire('fileuploadstart', {
            file: this.file
        });

        Y.Assert.areSame('STARTED', this.file.get('status'));

    }

}));


Y.Test.Runner.add(suite);

}, '@VERSION@', {
        requires: ['one-uploader']
});
