YUI.add('one-util-oops-test', function (Y) {

    var Assert = Y.Assert,
    suite;

    suite = new Y.Test.Suite('one-util-oops');

    suite.add(new Y.Test.Case({

        name: 'Oops tests',

        setUp : function () {
            Y.config.global.JSOOPS_URL = '';
        },

        tearDown: function () {
            Y.config.global.onerror = null;
        },

        "test instansiation": function () {
            Y.Assert.isInstanceOf(Y.U1.util.Oops, new Y.U1.util.Oops());
        },

        "a custom window.onerror handler is attached": function () {
            var ok,
                oops = new Y.U1.util.Oops({
                    globalErrorHandler: function () {
                        ok = true;
                    }
                });

            Y.config.global.onerror();
            ok ? Y.Assert.pass() : Y.Assert.fail('error handler did not execute');
        },

        "old handler is restored": function () {

            Y.config.global.onerror = function () {
                Y.Assert.pass();
            }

            var oops = new Y.U1.util.Oops({
                globalErrorHandler: function () {
                    Y.Assert.fail('oops error handler called instead');
                }
            });

            oops.destroy();

            Y.config.global.onerror();
        }

    }));

    suite.add(new Y.Test.Case({

        name: 'OopsReport tests',

        setUp : function () {
            this.report = new Y.U1.util.OopsReport({
                message: 'Error',
                url: 'fail.js',
                line: 42
            }, {
                reporter: 'u1.test',
                endPoint: '/foo',
                ioConfig: { xdr: { use: 'mock' } }
            });

            this.xhr = Y.Mock();
            Y.Mock.expect(this.xhr, {
                method: 'send',
                args: [Y.Mock.Value.Object, '/foo', Y.Mock.Value.Object]
            });
            this._createMockTransport(this.xhr);
        },

        tearDown: function () {
            delete this.report;
            delete this.xhr;
        },

        _createMockTransport: function (mockXHR) {
            Y.IO.transports.mock = function () {
                return mockXHR;
            }
        },

        "test instansiation": function () {
            Y.Assert.isInstanceOf(Y.U1.util.OopsReport, new Y.U1.util.OopsReport());
        },

        "test that send uses io": function () {

            Y.Mock.expect(this.xhr, {
                method: 'send',
                args: [Y.Mock.Value.Object, '/foo', Y.Mock.Value.Object],
                callCount: 1
            });

            this.report.send();
            Y.Mock.verify(this.xhr);
        },

        "send will pass along data": function () {

            this._createMockTransport({
                send: function (transaction, uri, config) {

                    var data = JSON.parse(config.data);

                    Y.ObjectAssert.ownsKey('reporter', data, 'Has data from sessionData');

                    Y.Assert.areSame('Error', data.message);
                    Y.Assert.areSame('fail.js', data.url);
                    Y.Assert.areSame(42, data.line);
                }
            });

            this.report.config.reporter = 'u1';

            this.report.send();

        },


        "test JSON stringification does not fail even if it encounters a cyclical reference": function () {

            this.report.config.sessionData = this.report;
            Y.Assert.isString(this.report._createReport());

        },


        "failed JSON stringification reports a oops report": function () {

            Y.Mock.expect(this.xhr, {
                method: 'send',
                args: [Y.Mock.Value.Object, '/foo', Y.Mock.Value.Object],

                callCount: 2
            });

            // make a cyclical ref
            this.report.config.sessionData = this.report;

            this.report.send();
            Y.Mock.verify(this.xhr);

        }

    }));

    Y.Test.Runner.add(suite);

}, '@VERSION@', {
    requires: ['one-util-oops', 'test']
});
