/*global current_folderid:true, current_path:true, hash_location:true */
/*global new_share_uri:true, snippet_uri:true, mkdir_uri:true undelete_uri:true undelete_msg:true undelete_success_msg:true */
/*global delete_udf_uri:true, delete_share_uri:true, publish_uri:true */
/*global delete_uri:true, can_write:true, can_share:true */
/*global current_folderid:true, storage_is_full:true, full:true  */
/*global above_quota_error_msg:true */
/*global UPDOWN_URL */

var ACTIONS = 'actions',
ALIGN = 'align',
BACKGROUND = 'background',
BODYCONTENT = 'bodyContent',
BUSY = 'one:busy',
CANCELLINK = 'cancelLink',
CLICK = 'click',
CREATEBUTTON = 'createButton',
DASHCONTENT = '-content',
DASHFOOTER = '-footer',
DASHHEADER = '-header',
DISABLED = 'disabled',
DISPLAY = 'display',
EMAIL = 'email',
ENTERKEYUP = 'up:13',
ERROR = 'one:error',
FILE = 'file',
FOOTERCONTENT = 'footerContent',
HEADERCONTENT = 'headerContent',
HTMLID = 'htmlID',
KEY = 'key',
NAMEFIELD = 'nameField',
NODEKEY = 'nodeKey',
NONE = 'none',
OVERLAY = 'overlay',
SHAREBUTTON = 'shareButton',
SHARENAME = 'shareName',
UPLOADBUTTON = 'uploadButton',
VALUE = 'value',
VALUECHANGE = VALUE + 'Change',
VISIBLE = 'visible',
VISIBLECHANGE = VISIBLE + 'Change',
WHITE = '#ffffff',
YELLOW = '#ffcccc',
FilesOverlay,
UploadOverlay,
NewFolderOverlay,
SharingOverlay,
FilesManager,
fileLocation,
historyHash,
delete_object,

u1 = Y.namespace('u1'),
U1 = Y.namespace('U1');

FilesOverlay = function() {};

FilesOverlay.prototype = {
    alignOverlay: function() {
        var POS = Y.WidgetPositionAlign;
        this.set(ALIGN, {
            node: '#main-nav',
            points: [POS.TC, POS.CC]
        });
    }
};

UploadOverlay = Y.Base.create(OVERLAY, Y.Overlay, [FilesOverlay], {

    initializer: function() {

        this.get('parent_key').set(VALUE, current_folderid); // global
        this.get('base').set(VALUE, '/files/' + hash_location); // global
        this.get('path').set(VALUE, '/' + current_path); // global

    },

    bindUI: function() {

        this.get(UPLOADBUTTON).on('click', this.uploadFile, this);
        this.get('cancelLinks').on('click', this.hide, this);
        this.after(VISIBLECHANGE, this.afterVisibleChange, this);
        this.get(FILE).after('change', this.afterFileChange, this);

        this.before(VISIBLECHANGE, this._visibleChange);

    },

    renderUI: function() {

        var contentID = '#upload-overlay',
            header = Y.one(contentID + DASHHEADER),
            body = Y.one(contentID + DASHCONTENT),
            footer = Y.one(contentID + DASHFOOTER);

        this.set(HEADERCONTENT, header);
        this.set(BODYCONTENT, body);
        this.set(FOOTERCONTENT, footer);
    },

    _useFilesUploader: function () {

        return Y.Uploader.TYPE === 'html5' && !Y.UA.ios;

    },

    _createFilesUploader: function () {

        Y.UploaderHTML5.SELECT_FILES_BUTTON = '<button type="button" class="cta secondary" role="button" aria-label="{selectButtonLabel}" ' +
             'tabindex="{tabIndex}">{selectButtonLabel}</button>';

        this.uploader = new U1.Widgets.FilesUploader({
            csrfmiddlewaretoken: YUI.Env.CSRF_TOKEN,
            current_folderid:       current_folderid,
            current_path:           current_path,
            fileFieldName:          'file',
            multipleFiles:          true,
            path:                   '/',
            simLimit:               2,
            simpleUploader:         '.files-simple-upload',
            swfURL:                 "/media/js/yui/uploader/assets/flashuploader.swf?t=" + Math.random(),
            timeout:                15,
            uploadURL:              UPDOWN_URL + "/upload/",
            withCredentials:        true
        });

        this.uploader.render('.files-multi-upload');

        this.uploader.after(['destroy', 'fullycomplete'], function () {
            this.hide();
            u1.refresh_snippet(current_path);
        }, this);

        this.set('width', '700px');
    },

    uploadFile: function() {

        this.hide();
        Y.Global.fire(BUSY);
        Y.io(UPDOWN_URL + '/upload/', {
            method: 'POST',
            form: {
                id: 'upload-overlay-form',
                upload: true,
                "public": this.get('publicCheckbox').get('checked')
            },
            on: {
                complete: Y.bind(this.onIOComplete, this),
                failure: function() {
                    Y.Global.fire(ERROR);
                }
            }
        });

    },

    _visibleChange: function (e) {

        // let one-uploader's destroy event dictate if we should hide this widget or not
        // cancel will call destroy, but make sure the user really wants to
        if(!e.newVal && this._useFilesUploader() && !this.uploader.get('destroyed')) {
            e.preventDefault();
            this.uploader.cancel();
        }

    },

    afterVisibleChange: function() {
        if (this.get(VISIBLE)) {
            this.alignOverlay();
            this.get(UPLOADBUTTON).set(DISABLED, true);

            // create the uploader
            // if it fails, keep the old uploader
            if (this._useFilesUploader()) {
                try {
                    this._createFilesUploader();
                } catch (e) {
                    Y.log(e, 'error', '_createFilesUploader');
                    void(0);
                }
            }

        }
    },

    afterFileChange: function(e) {
        var uploadButton = this.get(UPLOADBUTTON);
        if (e.target.get(VALUE).length === 0) {
            uploadButton.set(DISABLED, true);
        } else {
            uploadButton.set(DISABLED, false);
        }
    },

    onIOComplete: function() {
        var actions = this.get(ACTIONS),
            filename = this.get(FILE).get(VALUE),
            parts;

        actions.setStyle(DISPLAY, NONE);

        if (filename.match(/\\/)) {
            parts = filename.split('\\');
            filename = parts[parts.length - 1];
        }
        actions.setStyle(DISPLAY, 'block');
        this.get('publicCheckbox').set('checked', false);
        u1.refresh_snippet(current_path, filename);
    }

}, {
    ATTRS: {
        actions: { value: null },
        actionsContainer: { value: null },
        base: { value: null },
        cancelLinks:  { value: null },
        file: { value: null },
        parent_key: { value: null },
        path: { value: null },
        publicCheckbox: { value: null },
        render: { value: true },

        uploadButton: { value: null },
        visible: { value: false }
    },
    HTML_PARSER: {
        actions: '#upload-overlay-actions',
        actionsContainer: '#upload-overlay-actions-container',
        base: '#upload-overlay-base',
        cancelLinks: ['.upload-overlay-cancel'],
        file: '#upload-overlay-file',
        parent_key: '#upload-overlay-parent-key',
        path: '#upload-overlay-path',
        publicCheckbox: '#upload-overlay-public',
        uploadButton: '#upload-overlay-upload-button'
    }
});

u1.uploadOverlay = UploadOverlay;

NewFolderOverlay = Y.Base.create(OVERLAY, Y.Overlay, [FilesOverlay], {
    initializer: function() {
        this.get('parent').set(VALUE, current_folderid); // global?
    },
    renderUI: function() {
        var contentID = '#new-folder-overlay',
            header = Y.one(contentID + DASHHEADER),
            body = Y.one(contentID + DASHCONTENT),
            footer = Y.one(contentID + DASHFOOTER);

        header.remove();
        this.set(HEADERCONTENT, header);
        body.remove();
        this.set(BODYCONTENT, body);
        footer.remove();
        this.set(FOOTERCONTENT, footer);
    },
    bindUI: function() {
        this.get(CREATEBUTTON).on('click', this.createFolder, this);
        this.get(CANCELLINK).on('click', this.hide, this);
        this.get(NAMEFIELD).on(
            VALUECHANGE, this.afterNameValueChange, this);
        this.get(NAMEFIELD).on(KEY, this.onReturn, ENTERKEYUP, this);
        this.after(VISIBLECHANGE, this.afterVisibleChange, this);
    },
    createFolder: function() {
        this.hide();
        this.get(ACTIONS).setStyle(DISPLAY, NONE);
        Y.Global.fire(BUSY);
        Y.io(mkdir_uri, {
            method: 'POST',
            form: {id: 'new-folder-overlay-form' },
            on: {
                success: Y.bind(this.onIOSuccess, this),
                failure: function() { Y.Global.fire(ERROR); }
            }
        });
    },
    onIOSuccess: function() {
        this.get(ACTIONS).setStyle(DISPLAY, 'block');
        u1.refresh_snippet(current_path, this.get(NAMEFIELD).get(VALUE));
    },
    onReturn: function() {
        if (!this.get(CREATEBUTTON).get(DISABLED)) {
            this.createFolder();
        }
    },
    afterNameValueChange: function(e) {
        function checkBlacklist(str) {
            if (str === '.' || str === '..') { return true; }
            else { return (/[\0\/]/).test(str); }
        }
        var field = e.target,
            button = this.get(CREATEBUTTON),
            fieldValue = field.get(VALUE),
            disabled = false,
            background = WHITE;

        if (fieldValue.length === 0) {
            disabled = true;
        } else {
            if (checkBlacklist(fieldValue)) {
                background = YELLOW;
                disabled = true;
            }
        }
        field.setStyle(BACKGROUND, background);
        button.set(DISABLED, disabled);
    },
    afterVisibleChange: function() {
        if (this.get(VISIBLE)) {
            this.alignOverlay();
            var name = this.get(NAMEFIELD);
            name.set(VALUE, '');
            name.focus();
            this.get(CREATEBUTTON).set(DISABLED, true);
        }
    }
}, {
    ATTRS: {
        actions: { value: null },
        actionsContainer: { value: null },
        cancelLink: { value: null },
        createButton: { value: null },
        nameField: { value: null },
        render: { value: true },
        visible: { value: false },
        parent: { value: null },
        width: { value: '40%' }
    },
    HTML_PARSER: {
        actions: '#new-folder-overlay-actions',
        actionsContainer: '#new-folder-overlay-actions-container',
        createButton: '#new-folder-overlay-create-button',
        cancelLink: '#new-folder-overlay-cancel',
        nameField: '#new-folder-overlay-name',
        parent: '#new-folder-overlay-parent'
    }
});

SharingOverlay = Y.Base.create(OVERLAY, Y.Overlay, [FilesOverlay], {
    bindUI: function() {
        this.get(SHAREBUTTON).on('click', this.createShare, this);
        this.get(CANCELLINK).on('click', this.hide, this);
        this.after(VISIBLECHANGE, this.afterVisibleChange, this);

        this.get(EMAIL).after(VALUECHANGE, this.afterEmailValueChange, this);
        this.get(SHARENAME).after(VALUECHANGE, this.afterEmailValueChange, this);
        this.get(EMAIL).on(KEY, this.onReturn, ENTERKEYUP, this);
        this.get(SHARENAME).on(KEY, this.onReturn, ENTERKEYUP, this);
    },
    renderUI: function() {
        var contentID = '#sharing-overlay',
            header = Y.one(contentID + DASHHEADER),
            body = Y.one(contentID + DASHCONTENT),
            footer = Y.one(contentID + DASHFOOTER);

        header.remove();
        this.set(HEADERCONTENT, header);
        body.remove();
        this.set(BODYCONTENT, body);
        footer.remove();
        this.set(FOOTERCONTENT, footer);
    },
    afterVisibleChange: function() {
        if (this.get(VISIBLE)) {
            this.alignOverlay();
            this.get(EMAIL).set(VALUE, '');
            this.get(EMAIL).focus();
            this.get(SHAREBUTTON).set(DISABLED, true);
        }
    },
    afterEmailValueChange: function() {
        var emailField = this.get(EMAIL),
            shareNameField = this.get(SHARENAME),
            button = this.get(SHAREBUTTON),
            email = emailField.get(VALUE),
            share_name = shareNameField.get(VALUE),
            disabled = false,
            email_background = WHITE,
            share_name_background = WHITE;

        if (email.length === 0 || share_name.length === 0) {
            disabled = true;
        } else {
            if (share_name === '.' || share_name === '..' ||
                                     /[\0\/]/.test(share_name)) {
                share_name_background = YELLOW;
                disabled = true;
            }
            if (!/^.+@.+(\..+)+$/.test(email)) {
                email_background = YELLOW;
                disabled = true;
            }
        }
        button.set(DISABLED, disabled);
        emailField.setStyle(BACKGROUND, email_background);
        shareNameField.setStyle(BACKGROUND, share_name_background);
    },
    createShare: function() {
        this.hide();
        Y.Global.fire(BUSY);
        this.get(ACTIONS).setStyle(DISPLAY, NONE);

        var shareNodeKey = this.get(NODEKEY).get(VALUE),
            shareEmail = this.get(EMAIL).get(VALUE),
            shareName  = this.get(SHARENAME).get(VALUE),
            readOnly = this.get('readwrite').get(
                'checked') ? "false" : "true";

        Y.io(new_share_uri, {
            method: 'POST',
            data: {
                'nodekey': shareNodeKey,
                'offer_to_email': shareEmail,
                'share_name': shareName,
                'read_only': readOnly,
                'csrfmiddlewaretoken': YUI.Env.CSRF_TOKEN
            },
            on: {
                success: Y.bind(this.onIOSuccess, this)
            }
        });
    },
    onIOSuccess: function() {
            u1.refresh_snippet(
                current_path, this.get(HTMLID).get(VALUE));
            this.get(ACTIONS).setStyle(DISPLAY, 'block');
    },
    onReturn: function() {
        if (!this.get(SHAREBUTTON).get(DISABLED)) { this.createShare(); }
    },
    showForCurrentFolder: function() {
        var shareName = current_path.match(
            '([^/]*)/?$')[1] || 'My Storage';
        this.get(NODEKEY).set(VALUE, current_folderid);
        this.get(SHARENAME).set(VALUE, shareName);
        if (!current_path) {
            Y.one('#rootfolder-notice').addClass('shown');
        } else {
            Y.one('#rootfolder-notice').removeClass('shown');
        }
        this.show();
    },
    showForTableFolder: function(e) {
        var targetNodeID = e.target.get('id'),
            nodeID = targetNodeID.replace(
                /^fus(?:(.*)---___---)?(.*)-share/, '$1:$2'),
            nodeIDHtml = targetNodeID.replace(/^(.*)-share/, '$1'),
            // The anchor inside files-td-name has the folder's name
            shareName = Y.one('#' + nodeIDHtml).one(
                '.files-td-name a').get('innerHTML').split('/').pop();
        Y.one('#rootfolder-notice').removeClass('shown');
        this.get(HTMLID).set(VALUE, nodeIDHtml);
        this.get(NODEKEY).set(VALUE, nodeID);
        this.get(SHARENAME).set(VALUE, shareName);
        this.show();
    }
}, {
    ATTRS: {
        actions: { value: null },
        actionsContainer: { value: null },
        cancelLink: { value: null },
        email: { value: null },
        htmlID: { value: null },
        nodeKey: { value: null },
        readwrite: { value: null },
        render: { value: true },
        shareButton: { value: null },
        shareName: { value: null },
        visible: { value: false },
        width: { value: '40%' }
    },
    HTML_PARSER: {
        actions: '#sharing-overlay-actions',
        actionsContainer: '#sharing-overlay-actions-container',
        cancelLink: '#sharing-overlay-cancel',
        email: '#sharing-overlay-email',
        htmlID: '#sharing-overlay-html_id',
        nodeKey: '#sharing-overlay-nodekey',
        readwrite: '#sharing-overlay-readwrite',
        shareButton: '#sharing-overlay-share-button',
        shareName: '#sharing-overlay-name'
    }
});

FilesManager = Y.Base.create('filesManager', Y.Widget, [], {
    initializer: function() {
        var uploadOverlay = new UploadOverlay({
            srcNode: Y.one('#upload-overlay'),
            zIndex: 10
            }),
        newFolderOverlay = new NewFolderOverlay({
            srcNode: Y.one('#new-folder-overlay'),
            zIndex: 10
            }),
        sharingOverlay = new SharingOverlay({
            srcNode: Y.one('#sharing-overlay'),
            zIndex: 10
            });
        this.set('uploadOverlay', uploadOverlay);
        this.set('newFolderOverlay', newFolderOverlay);
        this.set('sharingOverlay', sharingOverlay);
        // We can't remove #overlays at this point as it's
        // included in the XHR response.
        Y.one('#overlays').setStyle('visibility', VISIBLE);
    },
    bindUI: function() {
        var uploadOverlay = this.get('uploadOverlay'),
            newFolderOverlay = this.get('newFolderOverlay'),
            undeleteButton = this.get('undeleteButton'),
            shareButton = this.get('shareThisFolderButton');

        // storage full, fire error message
        if(storage_is_full) {
            this.get(UPLOADBUTTON).on(
                'click', function () {
                    // This contains a string with HTML without user data so we turn off escaping.
                    Y.Global.fire(ERROR, { message: above_quota_error_msg, escapeHTML: false });
                });
        }
        // user have room for more stuffs, show uploadoverlay
        else {
            this.get(UPLOADBUTTON).on(
                'click', uploadOverlay.show, uploadOverlay);
        }
        this.get('newFolderButton').on(
            'click', newFolderOverlay.show, newFolderOverlay);
        if (can_share && shareButton) {
            shareButton.on('click', Y.bind(function() {
                this.get('sharingOverlay').showForCurrentFolder();},
                this));
        }

        // if we can undelete
        if(undeleteButton) {
            undeleteButton.on('click', this.undelete, this);
        }
    },

    showForTableFolder: function(e) {
        this.get('sharingOverlay').showForTableFolder(e);
    },

    undelete: function () {

        if(window.confirm(undelete_msg)) {

            Y.io(undelete_uri, {
                method: 'POST',
                    on: {
                        success: Y.bind(this.undeleteSuccess, this),
                        failure: function() { Y.Global.fire(ERROR); }
                    }
            });

        }
    },

    undeleteSuccess: function () {
        Y.Global.fire('one:info', {'message': undelete_success_msg });
        u1.refresh_snippet(current_path, null, true);
    }

}, {
    ATTRS: {
        uploadButton: { value: null },
        uploadOverlay: { value: null },
        newFolderButton: { value: null },
        newFolderOverlay: { value: null },
        shareThisFolderButton: { value: null },
        undeleteButton: { value: null },
        sharingOverlay: { value: null }
    },
    HTML_PARSER: {
        uploadButton: '#upload-file-button',
        newFolderButton: '#new-folder-button',
        shareThisFolderButton: '#share-current-folder-button',
        undeleteButton: '#undelete-files-button'
    }
});


u1.onRefreshSnippet = function(e){
    if (e && e.newVal !== "undefined") {
        u1.refresh_snippet(e.newVal);
    }
};

    u1.refresh_snippet = function(change_path, file_to_disclose, silent) {

        if(!silent) {
            Y.Global.fire(BUSY);
        }

        var snippet = Y.one('#files_tng'),
        uri = snippet_uri + change_path,
        extra_global_escapes = {
            "\\?": "%253F",
            "#": "%23",
            "%2F": "/"
        }, rx,
        data = '',
        prop;

        if (file_to_disclose) {
            data = 'file_to_disclose=' + file_to_disclose;
        }

        for (prop in extra_global_escapes) {
            rx = new RegExp(prop, 'g');
            uri = uri.replace(rx, extra_global_escapes[prop]);
        }

        Y.io(uri, {data: data, on: {
            success: function(request_id, o) {

                /*jshint evil:true */
                // webkit doesn't evaluate javascript in a XHR snippet. Workaround:
                can_write = o.responseText.match(/^can_write = true;$/m) !== null;
                can_share = o.responseText.match(/^can_share = true;$/m) !== null;
                current_path = o.responseText.match(/^current_path = (".*");$/m);
                current_path = current_path !== null ? eval(current_path[1]) : '';
                current_folderid = o.responseText.match(/^current_folderid = (".+");$/m);
                current_folderid = current_folderid !== null ? eval(current_folderid[1]) : '';
                snippet.setContent(o.responseText);
                Y.Global.fire('one:idle');
                u1.hookup(full);
                Y.on('contentready', function() {
                    if (file_to_disclose) {
                        var child = Y.one('[id="' + file_to_disclose + '"]').get(
                            'parentNode').next();
                            if (child) { child.scrollIntoView(); }
                    }
                }, '[id="' + file_to_disclose + '"]', Y);
            },

            failure: function () {
                Y.Global.fire(ERROR);
            }
        }
        });

    };

    u1.delete_file = function(item) {
        var node_id = item.currentTarget.get('id');
        delete_object(node_id, FILE);
    };

    u1.delete_current_dir = function(item) {
        var node_id = item.currentTarget.get('id');
        delete_object(node_id, 'directory');
    };

    delete_object = function(node_id, type) {
        var row = node_id.replace(/^(.*)-delete/, '$1'),
            row_node = Y.one("#" + row),
            row_name = row_node.one("td.files-td-name"),
            details = node_id.replace(/^(.*)-delete/, '$1-details'),
            oid = node_id.replace(/^fus(?:(.*)---___---)?(.*)-delete/, '$1:$2'),
            confirm_delete = window.confirm(
                'Are you sure you want to delete ' +
                    (type === 'directory' ? 'folder' : type) + ' ' +
                    row_name.get('id') + '?' +
                    '\n(Warning: This change will sync with all computers.)'),
            request;

        function onIOSuccess() {
            var row_node = Y.one("#" + row),
            details_node = Y.one("#" + details);

            details_node.remove(),
            row_node.remove();
            Y.fire('object:deleted', {type: type});
        }
        function onIOFailure() {
            var row_node = Y.one("#" + row);

            row_node.addClass('lazr-error');
        }

        if(confirm_delete) {
            // Enable a spinner on the object's row
            request = Y.io(delete_uri + oid, {
                method:'DELETE',
                on: {
                    success: onIOSuccess,
                    failure: onIOFailure
                }
            }
                          );
        }
    };

    u1.delete_udf = function(item) {
        var node_id = item.currentTarget.get('id'),
        oid = node_id.replace(
                /^fus(?:(.*)---___---)?(.*)-delete/, '$1:$2'),
        confirm_delete = window.confirm(
            'Are you sure you want to stop syncing this folder? ' +
                '\nThis will NOT delete the folder on your computer.'),
        request;

        function onIOSuccess() {
            u1.refresh_snippet(current_path);
        }
        if (confirm_delete) {
            request = Y.io(delete_udf_uri, {
                method: 'POST',
                data: {
                    'udf_id': oid,
                    'csrfmiddlewaretoken': YUI.Env.CSRF_TOKEN
                },
                on: {success: onIOSuccess}
            });
        }
    };

    u1.delete_share = function(item) {
        var confirm_delete = window.confirm('Are you sure you want to delete this share?'),
        node_id,
        oid,
        request;

        function onIOSuccess() {
            u1.refresh_snippet(current_path);
        }

        if(confirm_delete) {
            node_id = item.currentTarget.get('id');
            oid = node_id.replace(
                /^fus(?:(.*)---___---)?(.*)-delete/, '$1:$2');
            request = Y.io(delete_share_uri, {
                method: 'POST',
                data: {
                    'node_id': oid,
                    'csrfmiddlewaretoken': YUI.Env.CSRF_TOKEN
                },
                on: {success: onIOSuccess}
            });
        }
    };

    u1.publish_file = function(item) {
        var node_id = item.currentTarget.get('id'),
            oid = node_id.replace(/^fus(?:(.*)---___---)?(.*)-publish/, '$1:$2'),
            htmlID = node_id.replace(/^(.*)-publish/, '$1'),
            request;

        function onIOSuccess() {
            u1.refresh_snippet(current_path, htmlID);
        }
        request = Y.io(publish_uri + oid, {
            method: 'POST',
            data: {
                'is_public': 'True',
                'csrfmiddlewaretoken': YUI.Env.CSRF_TOKEN
            },
            on: {success: onIOSuccess}
        });
    };

    u1.stop_publishing_file = function(item) {
        var node_id = item.currentTarget.get('id'),
        oid = node_id.replace(/^fus(?:(.*)---___---)?(.*)-publish/, '$1:$2'),
        htmlID = node_id.replace(/^(.*)-publish/, '$1'),
        request;

        function onIOSuccess() {
            u1.refresh_snippet(current_path, htmlID);
        }
        request = Y.io(publish_uri + oid, {
            method: 'POST',
            data: {
                'is_public': 'False',
                'csrfmiddlewaretoken': YUI.Env.CSRF_TOKEN
            },
            on: {success: onIOSuccess}
        });
    };

u1.stop_sharing = function(item) {
    var confirm_delete = window.confirm('Are you sure you want to stop sharing this folder?'),
    request;

    function onIOSuccess() {
        // We may be doing this from the actual folder, not the row
        try {
            var node_id = item.currentTarget.ancestor('tr').get('id'),
            htmlID = node_id.replace(/^(.*)-details/, '$1');
            u1.refresh_snippet(current_path, htmlID);
        } catch(err) {
            u1.refresh_snippet(current_path, htmlID);
        }
    }

    if (confirm_delete) {
        request = Y.io(delete_share_uri, {
            method: 'POST',
            data: {
                'share_id': item.currentTarget.get('id'),
                'csrfmiddlewaretoken': YUI.Env.CSRF_TOKEN
            },
            on: {success: onIOSuccess}
        });
    }
};

u1.hookup = function(full) {
    var file_rows = Y.all('.file'),
        directory_rows = Y.all('.directory'),
        shares_rows = Y.all('.share'),
        udfs_rows = Y.all('.udf'),
        delete_file_links = Y.all('.u1-files-delete-link'),
        delete_udf_links = Y.all('.u1-udf-delete-link'),
        delete_share_links = Y.all('.u1-share-delete-link'),
        delete_dir_links = Y.all('.u1-dirs-delete-link'),
        publish_file_links = Y.all('.u1-files-publish-link'),
        stop_publishing_file_links = Y.all('.u1-files-stop-publishing-link'),
        breadcrumbs = Y.all('.breadcrumb'),
        manager;

     function toggle_visibility(item) {
        // Get the TR
        var row_node = item.currentTarget.get('parentNode'),
            node_id = row_node.get('id'),
            node_details = Y.one('#' + node_id + '-details');

            if(!row_node.test('.inprogress')) {
                if (node_details.hasClass('lazr-closed')) {
                    node_details.replaceClass('lazr-closed', 'lazr-opened');
                } else {
                    node_details.replaceClass('lazr-opened', 'lazr-closed');
                }
            }

       }

    function ajaxify_crumb(e){
        e.preventDefault();
        var link_node = e.currentTarget,
            anchor = link_node,
            url = anchor.getAttribute('href').substr(7);

        anchor.removeAttribute('href');
        current_path = url;
        u1.location_update(current_path);
    }

    function ajaxify_row(tr_node){
        function navigate(e, url, node_id){
            current_folderid = node_id;
            current_path = url;
            e.preventDefault();
            u1.location_update(current_path);
        }

        var anchor = tr_node.one('a'),
        node_id = tr_node.getAttribute('id').replace(/^fus(?:(.*)---___---)?(.*)/, '$1:$2'),
        url = anchor.getAttribute('href').substr(7);

        anchor.removeAttribute('href');
        Y.on(CLICK, navigate, tr_node.one('.files-td-name'), '', url, node_id);
        Y.on(CLICK, navigate, tr_node.one('.files-td-size'), '', url, node_id);
        Y.on(CLICK, navigate, tr_node.one('.files-td-date'), '', url, node_id);
    }

    function file_row_clickable(item){
        var file_link,
        url;

        function navigate(e, url){
            window.open(url);
        }
        function preventDoubleLink(e){
            e.preventDefault();
        }
        if(!item.test('.inprogress')) {
            file_link = item.one('a');
            url = file_link.getAttribute('href');
            Y.on(CLICK, preventDoubleLink, file_link);
            Y.on(CLICK, navigate, item.one('.files-td-name'), '', url);
            Y.on(CLICK, navigate, item.one('.files-td-size'), '', url);
            Y.on(CLICK, navigate, item.one('.files-td-date'), '', url);
        }
    }

    file_rows.each(function(item){
        file_row_clickable(item);
    });
    directory_rows.each(function(item){
        ajaxify_row(item);
    });
    udfs_rows.each(function(item){
        ajaxify_row(item);
    });
    shares_rows.each(function(item){
        ajaxify_row(item);
    });
    breadcrumbs.each(function(item){
        Y.on(CLICK, ajaxify_crumb, item);
    });

    if (full) {
        if (can_write) {
            manager = new FilesManager({
                srcNode: Y.one('#files-ui')
            });
            manager.render();

            Y.on(CLICK, toggle_visibility, Y.all('.expand-row-link'));
            Y.on(CLICK, u1.delete_current_dir, delete_dir_links);
            Y.on(CLICK, u1.delete_file, delete_file_links);
            Y.on(CLICK, u1.delete_udf, delete_udf_links);
            Y.on(CLICK, u1.delete_share, delete_share_links);
            Y.on(CLICK, u1.publish_file, publish_file_links);
            Y.on(CLICK, u1.stop_publishing_file, stop_publishing_file_links);
            Y.on(CLICK, Y.bind(manager.showForTableFolder, manager), Y.all('.u1-dirs-share-link'));
            Y.on(CLICK, u1.stop_sharing, Y.all('.folder-stop-sharing'));
        }
    }
    };

    historyHash = new Y.HistoryHash();
    u1.location_update = function(change_path){
        historyHash.add({f: (change_path)});
    };
    historyHash.on('fChange', Y.u1.onRefreshSnippet);

    Y.on('object:deleted', function(e) {
             // Check to see the number of tr.<object-type> elements there are.
             // If 0: remove the header-<object> element
             var type = e.type,
             trs_left = Y.all('tr.' + type);
             if (trs_left._nodes.length === 0) {
                 Y.one('#header-' + type).remove();
             }
         });

fileLocation = historyHash.get('f');

if(fileLocation && fileLocation !== '/'){
    Y.u1.location_update(hash_location.substr(3));
    // For some reason, at this stage, refresh_snippet is not called
    // from the Y.History.navigate
    Y.u1.refresh_snippet(hash_location.substr(3));
}
